SystemOrganization addCategory: #'SomethingIt-VDM-Process'!
SystemOrganization addCategory: #'SomethingIt-VDM-Specification'!
SystemOrganization addCategory: #'SomethingIt-VDM-Data'!
SystemOrganization addCategory: #'SomethingIt-VDM-UI'!
SystemOrganization addCategory: #'SomethingIt-VDM-Test'!

StringHolder subclass: #SIVDMInspectiveBrowser
	instanceVariableNames: 'vdm state specification stateIndicated fieldListPane valuePane sectionListPane identifierListPane codePane selectedModule selectedField selectedSection selectedIdentifier code timeOfLastStateUpdate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-UI'!

!SIVDMInspectiveBrowser class methodsFor: 'class access' stamp: 'tomooda 2/26/2013 14:22'!
evaluatorClass
	^ SIVDMEvaluator! !

!SIVDMInspectiveBrowser class methodsFor: 'examples' stamp: 'tomooda 2/26/2013 14:10'!
exampleFizzbuzz
	"SIVDMInspectiveBrowser exampleFizzbuzz"

	| fizzbuzzSpec browser count sayFizz sayBuzz sayFizzBuzz sayNumber window font fizzButton buzzButton fizzbuzzButton numberButton |
	fizzbuzzSpec := SIVDM specification: '
/*
The FizzBuzz game counts numbers up,
saying Fizz when the number is divisible by 3,
Buzz when the number is divisible by 5,
FizzBuzz when the number is divisible by 15,
otherwise the number.
*/
state counter of
  count : int
init
  -- the game starts with "1".
  s == s = mk_counter(1)
end

functions
  -- say fizz when divisible by 3
  fizz : int -> seq of char
  fizz(x) == if x mod 3 = 0 then "fizz" else "";

  -- say buzz when divisible by 5
  buzz : int -> seq of char
  buzz(x) == if x mod 5 = 0 then "buzz" else "";

  -- say fizzbuzz when divisible by 15, otherwise the number.
  fizzbuzz : int -> token
  fizzbuzz(x) == let fb = fizz(x) ^ buzz(x) in if fb = "" then mk_token(x) else mk_token(fb);

operations
  -- A player says "fizz". Answers whether or not it is legitimate call.
  sayFizz : () ==> bool
  sayFizz() == if fizzbuzz(count) = mk_token("fizz") then (count := count + 1; return true) else return false; 
  -- A player says "buzz". Answers whether or not it is legitimate call.
  sayBuzz : () ==> bool
  sayBuzz() == if fizzbuzz(count) = mk_token("buzz") then (count := count + 1; return true) else return false; 
  -- A player says "fizzbuzz". Answers whether or not it is legitimate call.
  sayFizzBuzz : () ==> bool
  sayFizzBuzz() == if fizzbuzz(count) = mk_token("fizzbuzz") then (count := count + 1; return true) else return false; 
  -- A player says a number. Answers whether or not it is legitimate call.
  sayNumber : () ==> bool
  sayNumber() == if fizzbuzz(count) = mk_token(count) then (count := count + 1; return true) else return false; 

'.
	browser := SIVDMInspectiveBrowser openOn: fizzbuzzSpec withLabel: 'FizzBuzz spec'.
	sayFizz := 'sayFizz' asVDMOperationIn: fizzbuzzSpec.
	sayBuzz := 'sayBuzz' asVDMOperationIn: fizzbuzzSpec.
	sayFizzBuzz := 'sayFizzBuzz' asVDMOperationIn: fizzbuzzSpec.
	sayNumber := 'sayNumber' asVDMOperationIn: fizzbuzzSpec.
	count := 'count' asVDMVariableIn: fizzbuzzSpec.
	font := TextStyle default fontOfSize: 36.
	window := Morph new.
	window extent: 150 @ 150.
	window color: Color white.
	window changeTableLayout.
	window listDirection: #topToBottom.
	window vResizing: #shrinkWrap.
	numberButton := SimpleButtonMorph newWithLabel: count value printString.
	numberButton label: numberButton label font: font.
	numberButton
		target: [ 
			sayNumber value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	numberButton actionSelector: #value.
	numberButton hResizing: #spaceFill.
	window addMorphBack: numberButton.
	fizzButton := SimpleButtonMorph newWithLabel: 'Fizz'.
	fizzButton label: fizzButton label font: font.
	fizzButton
		target: [ 
			sayFizz value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	fizzButton actionSelector: #value.
	fizzButton hResizing: #spaceFill.
	window addMorphBack: fizzButton.
	buzzButton := SimpleButtonMorph newWithLabel: 'Buzz'.
	buzzButton label: buzzButton label font: font.
	buzzButton
		target: [ 
			sayBuzz value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	buzzButton actionSelector: #value.
	buzzButton hResizing: #spaceFill.
	window addMorphBack: buzzButton.
	fizzbuzzButton := SimpleButtonMorph newWithLabel: 'FizzBuzz'.
	fizzbuzzButton label: fizzbuzzButton label font: font.
	fizzbuzzButton
		target: [ 
			sayFizzBuzz value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	fizzbuzzButton actionSelector: #value.
	fizzbuzzButton hResizing: #spaceFill.
	window addMorphBack: fizzbuzzButton.
	window openInWindow.
	^ fizzbuzzSpec! !

!SIVDMInspectiveBrowser class methodsFor: 'examples' stamp: 'tomooda 2/26/2013 14:12'!
exampleGravityByOperation
	"SIVDMInspectiveBrowser exampleGravityByOperation"
	| vdmj  step positions |
	vdmj := SIVDM specification: '
module Point
exports all
definitions
types
  Point :: x : real
           y : real;
functions
  plus : Point * Point -> Point
  plus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1+x2, y1+y2);

  minus : Point * Point -> Point
  minus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1-x2, y1-y2);

  neg : Point -> Point
  neg(mk_Point(x,y)) == mk_Point(-x, -y);

  scale : real * Point -> Point
  scale(a, mk_Point(x,y)) == mk_Point(a*x, a*y);

  dist2 : Point -> real
  dist2(mk_Point(x,y)) == x * x + y * y;

  dist : Point -> real
  dist(p) == dist2(p) ** 0.5;
end Point

module Gravity
exports all
imports from Point
    types
        Point renamed Point;
    functions
        plus renamed plus;
        minus renamed minus;
        neg renamed neg;
        scale renamed scale;
        dist2 renamed dist2;
        dist renamed dist
definitions
values
  gmn = 10.0;
functions
  total : seq of Point -> Point
  total(ps) == cases ps :
    [] -> mk_Point(0.0,0.0),
    others -> plus(hd ps, total(tl ps))
    end;

  gravity1 : Point * Point -> Point
  gravity1(p1, p2) ==
    let
      v = minus(p2, p1),
      d2 = dist2(v)
    in
      if d2 < 400.0
      then mk_Point(0.0, 0.0)
      else scale(gmn/(d2**1.5), v);

  gravity : seq of Point -> seq of Point
  gravity(ps) == [total([gravity1(ps(i1), ps(i2)) | i2 in set {1,...,len ps}]) | i1 in set {1,...,len ps}];
end Gravity

module ParticlesSimulation
exports all
imports
    from Point
        types
            Point renamed Point;
        functions
            plus renamed plus,
    from Gravity
        functions
            gravity renamed gravity

definitions
state Particles of
  positions : seq of Point
  velocities : seq of Point
inv
  mk_Particles(positions, velocities) == len positions = len velocities
init
  particles == particles = mk_Particles([], [])
end
operations
  Step : () ==> ()
  Step() == (
    velocities := (let forces = gravity(positions) in [plus(velocities(i), forces(i)) | i in set {1,...,len forces}]);
    positions := [plus(positions(i), velocities(i)) | i in set {1,...,len velocities}]);
  GetPositions : () ==> seq of Point
  GetPositions() == return positions;
  Init : seq of Point * seq of Point ==> ()
  Init(ps, vs) == atomic(positions := ps; velocities := vs);
end ParticlesSimulation
'.
	SIVDMInspectiveBrowser openOn: vdmj withLabel: 'Gravity spec'.
	vdmj printer
		addClassMap: #Point
		to: [:point :vdmStream | 
			vdmStream nextPutAll: 'mk_Point`Point('.
			point x printOn: vdmStream.
			vdmStream nextPut: $,.
			point y printOn: vdmStream.
			vdmStream nextPut: $)].
	vdmj parser
		addAlgebraicDataMap: 'mk_Point'
		to: [:constructor :args | args first @ args second].
	vdmj parser
		addAlgebraicDataMap: 'mk_Point`Point'
		to: [:constructor :args | args first @ args second].
	('ParticlesSimulation`Init' asVDMOperationIn: vdmj)
		value: {100.0 @ 250.0. 250.0 @ 250.0. 250.0 @ 350.0} asOrderedCollection
		value: {0.0 @ 0.0. 0.0 @ 0.0. 0.0 @ 0.0} asOrderedCollection.
	step := 'ParticlesSimulation`Step' asVDMOperationIn: vdmj.
	positions := 'ParticlesSimulation`GetPositions' asVDMOperationIn: vdmj.
	[| form morph window colors |
	form := Form extent: 500 @ 500 depth: 32.
	morph := form asMorph.
	window := morph openInWindowLabeled: 'Gravity by VDM operations'.
	colors := (1 to: 3)
				collect: [:i | Color
						h: i * 360.0 / 10
						s: 1.0
						v: 1.0].
	[window owner notNil]
		whileTrue: [| canvas points |
			(morph bounds containsPoint: ActiveHand position)
				ifTrue: [step value].
			points := positions value.
			canvas := form getCanvas.
			canvas fillColor: Color white.
			1
				to: points size
				do: [:index | 
					| point color |
					point := points at: index.
					color := colors
								at: index
								ifAbsent: [Color black].
					canvas
						fillOval: (point - 4 extent: 8 @ 8)
						color: color].
			canvas flush.
			morph changed.
			(Delay forMilliseconds: 10) wait]]
		forkAt: Processor userBackgroundPriority.
	^ vdmj! !

!SIVDMInspectiveBrowser class methodsFor: 'instance creation' stamp: 'tomooda 2/26/2013 13:14'!
inspect: anSIVDM 
	^ self new vdmj: anSIVDM;
		 yourself! !

!SIVDMInspectiveBrowser class methodsFor: 'instance creation' stamp: 'tomooda 11/16/2012 11:31'!
openOn: anSIVDM 
	^ self openOn: anSIVDM withLabel: anSIVDM defaultLabelForInspector! !

!SIVDMInspectiveBrowser class methodsFor: 'instance creation' stamp: 'tomooda 11/16/2012 11:31'!
openOn: anSIVDM withLabel: label 
	^ ToolBuilder
		open: (self inspect: anSIVDM)
		label: label! !

!SIVDMInspectiveBrowser methodsFor: 'value pane' stamp: 'tomooda 2/26/2013 13:06'!
accept: aTextOrString 
	selectedModule
		ifNil: [^ false]
		ifNotNil: [:module | selectedField
				ifNil: [vdmj
						stateAt: module
						put: (Compiler evaluate: aTextOrString asString)]
				ifNotNil: [(state
						detect: [:fieldAndValue | fieldAndValue key = selectedField]
						ifNone: [^ false])
						value: (vdmj
								plainEvaluate: aTextOrString asString
								ifError: [:msg | 
									SINotifier notify: msg.
									^ false]).
					vdmj stateAt: module put: state.
					self updateState]].
	self updateContents.
	^ true! !

!SIVDMInspectiveBrowser methodsFor: 'code pane' stamp: 'tomooda 12/4/2012 09:50'!
acceptCode: aString notifying: aMorph 
	| source |
	source := aString asString.
	specification
		ifNotNil: [(specification
				moduleAt: self selectedModule
				ifAbsent: [])
				ifNotNil: [:module | 
					self selectedSection = 'exports'
						ifTrue: [self selectedIdentifier = 'types'
								ifTrue: [^ (module typeExport: source)
										ifNotNil: ['types']].
							self selectedIdentifier = 'values'
								ifTrue: [^ (module valueExport: source)
										ifNotNil: ['values']].
							self selectedIdentifier = 'functions'
								ifTrue: [^ (module functionExport: source)
										ifNotNil: ['functions']].
							self selectedIdentifier = 'operations'
								ifTrue: [^ (module operationExport: source)
										ifNotNil: ['operations']].
							^ false].
					self selectedSection = 'imports'
						ifTrue: [self selectedIdentifier = 'types'
								ifTrue: [^ (module typeImport: source)
										ifNotNil: ['types']].
							self selectedIdentifier = 'values'
								ifTrue: [^ (module valueImport: source)
										ifNotNil: ['values']].
							self selectedIdentifier = 'functions'
								ifTrue: [^ (module functionImport: source)
										ifNotNil: ['functions']].
							self selectedIdentifier = 'operations'
								ifTrue: [^ (module operationImport: source)
										ifNotNil: ['operations']].
							^ false].
					self selectedSection = 'types'
						ifTrue: [| identifier | ^ (module typeAt: (identifier := self getName: source) put: source)
								ifNotNil: [identifier]].
					self selectedSection = 'values'
						ifTrue: [| identifier | ^ (module valueAt: (identifier := self getName: source) put: source)
								ifNotNil: [identifier]].
					self selectedSection = 'functions'
						ifTrue: [| identifier | ^ (module functionAt: (identifier := self getName: source) put: source)
								ifNotNil: [identifier]].
					self selectedSection = 'state'
						ifTrue: [| identifier | ^ (module stateAt: (identifier := self getName: source) put: source)
								ifNotNil: [identifier]].
					self selectedSection = 'operations'
						ifTrue: [| identifier | ^ (module operationAt: (identifier := self getName: source) put: source)
								ifNotNil: [identifier]]]].
	^ false! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 12/5/2012 07:56'!
buildCodePaneWith: builder
	| textSpec |
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		name: #codePane;
		getText: #code; 
		setText: #code:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		askBeforeDiscardingEdits: true.
	^textSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 2/26/2013 13:51'!
buildFieldListWith: builder 
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec model: self;
	      name: #fieldListPane;
		 list: #fieldList;
		 getIndex: #fieldSelectionIndex;
		 setIndex: #fieldSelectionIndex:.
	^ listSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 12/5/2012 07:48'!
buildIdentifierListWith: builder 
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec model: self;
		 name: #identifierListPane;
		 list: #identifierList;
		 getIndex: #identifierSelectionIndex;
		 setIndex: #identifierSelectionIndex:;
		 menu: #identifierListMenu:;
		 dragItem: #dragFromIdentifierList:;
		 dropItem: #dropOnIdentifierList:at:;
		 keyPress: #identifierListKey:.
	^ listSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 2/26/2013 13:44'!
buildInspectorPanelWith: builder
	"Build the instance/comment/class switch"
	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec name: #inspectorPanel.
	panelSpec children: OrderedCollection new.
	panelSpec children addLast: ((self buildFieldListWith: builder) frame: (0@0 corner: 1@0.5); yourself).
	panelSpec children addLast: ((self buildValuePaneWith: builder) frame: (0@0.5 corner: 1@1); yourself).
	^panelSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 11/19/2012 13:59'!
buildMiddlePanelWith: builder 
	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	panelSpec children
		add: ((self buildFieldListWith: builder)
				frame: (LayoutFrame
						fractions: (0 @ 0 corner: 1 @ 1))).
	panelSpec children
		add: ((self buildSectionListWith: builder)
				frame: (LayoutFrame
						fractions: (0 @ 0 corner: 1 @ 1))).
	^ panelSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 11/16/2012 11:16'!
buildModuleListWith: builder

	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #moduleList; 
		getIndex: #moduleSelectionIndex; 
		setIndex: #moduleSelectionIndex:; 
		menu: #moduleListMenu:; 
		dragItem: #dragFromModuleList:;
		dropItem: #dropOnModuleList:at:;
		keyPress: #inspectorKey:from:.
	^listSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 11/19/2012 14:00'!
buildRightPanelWith: builder 
	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	panelSpec children
		add: ((self buildValuePaneWith: builder)
				frame: (LayoutFrame
						fractions: (0 @ 0 corner: 1 @ 1))).
	panelSpec children
		add: ((self buildIdentifierListWith: builder)
				frame: (LayoutFrame
						fractions: (0 @ 0 corner: 1 @ 1))).
	^ panelSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 11/19/2012 13:54'!
buildSectionListWith: builder 
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec model: self;
		 name: #sectionListPane;
		 list: #sectionList;
		 getIndex: #sectionSelectionIndex;
		 setIndex: #sectionSelectionIndex:;
		 menu: #sectionListMenu:;
		 dragItem: #dragFromSectionList:;
		 dropItem: #dropOnSectionList:at:;
		 keyPress: #inspectorKey:from:.
	^ listSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 2/26/2013 13:32'!
buildSwitchesWith: builder
	"Build the instance/comment/class switch"
	| panelSpec buttonSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.

	buttonSpec := builder pluggableButtonSpec new.
	buttonSpec 
			model: self;
			label: 'state'; 
			state: #stateIndicated; 
			action: #indicateState;
			frame: (0@0 corner: 0.5@1).
	panelSpec children addLast: buttonSpec.

	buttonSpec := builder pluggableButtonSpec new.
	buttonSpec 
			model: self;
			label: 'source'; 
			state: #sourceIndicated; 
			action: #indicateSource;
			frame: (0.5@0 corner: 1@1).
	panelSpec children addLast: buttonSpec.

	^panelSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 11/19/2012 13:56'!
buildValuePaneWith: builder 
	| textSpec |
	textSpec := builder pluggableTextSpec new.
	textSpec model: self;
		 name: #valuePane;
		 getText: #contents;
		 setText: #accept:;
		 selection: #contentsSelection;
		 menu: #valuePaneMenu:shifted:.
	^ textSpec! !

!SIVDMInspectiveBrowser methodsFor: 'toolbuilder' stamp: 'tomooda 2/26/2013 13:43'!
buildWith: builder
	| windowSpec window |
	windowSpec := self buildWindowWith: builder specs: {
		(0@0 corner: 0.3@0.7)  -> [self buildModuleListWith: builder].
		(0@0.7 corner: 0.3@1) -> [self buildSwitchesWith: builder].
		(0.3@0 corner: 1@1) -> [self buildCodePaneWith: builder].
		(0.3@0 corner: 1@1) -> [self buildInspectorPanelWith: builder].
	}.
	window := builder build: windowSpec.
	inspectorPanel := builder widgetAt: #inspectorPanel.
	codePane := builder widgetAt: #codePane.
	^ window! !

!SIVDMInspectiveBrowser methodsFor: 'code pane' stamp: 'tomooda 2/26/2013 13:08'!
code
	| section module |
	vdmj
		ifNil: [^ String new].
	module := self selectedModule.
	module
		ifNil: [^ vdmj specification].
	section := ((self modulePositionsFrom: vdmj specification)
				detect: [:assoc | assoc key = module]
				ifNone: [^ String new]) value.
	^ vdmj specification copyFrom: section first to: section last! !

!SIVDMInspectiveBrowser methodsFor: 'code pane' stamp: 'tomooda 2/26/2013 13:25'!
code: aString notifying: aMorph 
	vdmj
		ifNotNil: [| newModules oldModules source |
			newModules := Dictionary withAll: (self modulePositionsFrom: aString).
			oldModules := self modulePositionsFrom: vdmj specification.
			source := String new writeStream.
			source nextPutAll: (vdmj specification copyFrom: 1 to: oldModules first value first - 1).
			oldModules do: [ :assoc |
				(newModules includesKey: assoc key)
					ifTrue: [
						| section |
						section := newModules at: assoc key.
						source nextPutAll: (aString copyFrom: section first to: section last).
						newModules removeKey: assoc key]
					ifFalse: [
						| section |
						section := assoc value.
						source nextPutAll: (vdmj specification copyFrom: section first to: section last)]].
			(newModules associations asSortedCollection: [ :less :more | less value first < more value first]) do: [ :assoc |
				| section |
				section := assoc value.
				source nextPutAll: (aString copyFrom: section first to: section last)].
			vdmj
				swapSpecification: source contents
				ifError: [:msg | 
					SINotifier notify: msg.
					^ false].
			self changed: #moduleList.
			self updateState.
			^ true].
	^ false! !

!SIVDMInspectiveBrowser methodsFor: 'code pane' stamp: 'tomooda 2/26/2013 13:08'!
codePaneDoIt
	self doItIn: codePane! !

!SIVDMInspectiveBrowser methodsFor: 'code pane' stamp: 'tomooda 2/26/2013 13:09'!
codePaneMenu: aMenu shifted: aBoolean
	aMenu addList: (aBoolean = true 
		ifTrue:[{
			#-.
			{'save contents to file...' translated.		#saveContentsInFile}.
			{'send contents to printer' translated.		#sendContentsToPrinter}.
			{'printer setup' translated.				#printerSetup}.
			#-.
		}.]
		ifFalse:[{
			{'set font... (k)' translated.				#offerFontMenu}.
			{'set style... (K)' translated.				#changeStyle}.
			{'set alignment... (u)' translated.		#chooseAlignment}.
			#-.
			{'find...(f)' translated.					#find}.
			{'find again (g)' translated.				#findAgain}.
			{'set search string (h)' translated.		#setSearchString}.
			#-.
			{'do again (j)' translated.				#again}.
			{'undo (z)' translated.					#undo}.
			#-.
			{'copy (c)' translated.					#copySelection}.
			{'cut (x)' translated.						#cut}.
			{'paste (v)' translated.					#paste}.
			{'paste...' translated.					#pasteRecent}.
			#-.
			{'do it (d)' translated.		#codePaneDoIt}.			     
			{'print it (p)' translated.		#codePanePrintIt}.
			#-.
			{'accept (s)' translated.					#accept}.
			{'cancel (l)' translated.					#cancel}.
			#-.
			{'copy html' translated.					#copyHtml}.
			#-.
			{'more...' translated.					#shiftedTextPaneMenuRequest}.
		}]).
	^aMenu! !

!SIVDMInspectiveBrowser methodsFor: 'code pane' stamp: 'tomooda 2/26/2013 13:09'!
codePanePrintIt
	^ self printItIn: codePane! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 11/20/2012 10:44'!
doItIn: aPluggableTextMorph 
	valuePane
		handleEdit: [| editor error |
			editor := aPluggableTextMorph textMorph editor.
			editor
				lineSelectAndEmptyCheck: [^ nil].
			error := nil.
			self
				evaluate: editor selectionAsStream upToEnd
				ifError: [:msg | error := msg].
			error
				ifNotNil: [| selectionInterval |
					selectionInterval := editor selectionInterval.
					aPluggableTextMorph textMorph installEditorToReplace: editor.
					aPluggableTextMorph textMorph
						handleEdit: [editor afterSelectionInsertAndSelect: error].
					selectionInterval := editor selectionInterval.
					aPluggableTextMorph textMorph editor selectFrom: selectionInterval first to: selectionInterval last.
					aPluggableTextMorph scrollSelectionIntoView]]! !

!SIVDMInspectiveBrowser methodsFor: 'evaluation' stamp: 'tomooda 12/5/2012 08:05'!
doItReceiver
	^ self! !

!SIVDMInspectiveBrowser methodsFor: 'evaluation' stamp: 'tomooda 2/26/2013 13:39'!
dotItReceiver
	^ self! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 11/20/2012 09:54'!
evaluate: aString 
	^ self evaluate: aString ifError: [ :msg | msg ]! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 2/26/2013 13:32'!
evaluate: aString ifError: errorBlock 
	vdmj
		critical: [
			vdmj ensureStartupProcess.
			vdmj send: 'default '
					, (self selectedModule
							ifNil: [vdmj globalModuleName]).
			vdmj readUpToPrompt.
			[vdmj send: 'print ' , aString.
			^ vdmj
				expression: vdmj readUpToPrompt
				ifError: [:msg | errorBlock value: msg]]
				ensure: [vdmj send: 'default ' , vdmj globalModuleName.
					vdmj readUpToPrompt]]! !

!SIVDMInspectiveBrowser methodsFor: 'field list' stamp: 'tomooda 11/16/2012 13:34'!
fieldList
	^ state
		ifNil: [Array new]
		ifNotNil: [:s | s
				collect: [:fieldAndValue | fieldAndValue key]]! !

!SIVDMInspectiveBrowser methodsFor: 'field list' stamp: 'tomooda 11/16/2012 13:59'!
fieldSelectionIndex
	^ self fieldList indexOf: self selectedField! !

!SIVDMInspectiveBrowser methodsFor: 'field list' stamp: 'tomooda 11/16/2012 14:08'!
fieldSelectionIndex: anInteger
	self selectFieldNamed: (self fieldList at: anInteger ifAbsent: [])! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 12/4/2012 09:36'!
getName: aString 
	| vdmj source name |
	vdmj := SIVDM new.
	source := (vdmj whiten: aString) readStream.
	name := String new writeStream.
	vdmj skipSeparators: source ifAtEnd: [^nil].
	[| char |
	source atEnd ifTrue: [^nil].
	char := source next.
	(char isSeparator
			or: [char = $=
					or: [char = $:]])
		ifTrue: [name := name contents.
			name ~= 'state'
				ifTrue: [^ name].
			name := String new writeStream.
			[source peek isSeparator]
				whileTrue: [source next]]
		ifFalse: [name nextPut: char]] repeat! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list' stamp: 'tomooda 12/3/2012 13:03'!
identifierList
	specification
		ifNotNil: [(specification
				moduleAt: self selectedModule
				ifAbsent: [])
				ifNotNil: [:module | 
					self selectedSection = 'exports'
						ifTrue: [^ #('types' 'values' 'functions' 'operations')].
					self selectedSection = 'imports'
						ifTrue: [^ #('types' 'values' 'functions' 'operations')].
					self selectedSection = 'types'
						ifTrue: [^ module typeNames copy].
					self selectedSection = 'values'
						ifTrue: [^ module valueNames copy].
					self selectedSection = 'functions'
						ifTrue: [^ module functionNames copy].
					self selectedSection = 'state'
						ifTrue: [^ module stateName
								ifNil: [Array new]
								ifNotNil: [:state | Array with: state]].
					self selectedSection = 'operations'
						ifTrue: [^ module operationNames copy]]].
	^ Array new! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list menu' stamp: 'tomooda 12/5/2012 07:49'!
identifierListKey: aCharacter 
	aCharacter = $x
		ifTrue: [^ self removeIdentifier].
	^ false! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list menu' stamp: 'tomooda 12/4/2012 10:43'!
identifierListMenu: aMenu
	(Dictionary new at: 'types' put: 'type';
		 at: 'values' put: 'value';
		 at: 'functions' put: 'function';
		 at: 'state' put: 'state';
		 at: 'operations' put: 'operation';
		
		at: self selectedSection
		ifAbsent: [])
		ifNotNil: [:sectionName | aMenu
				addList: ({{('remove ' , sectionName) translated. #removeIdentifier}})].
	^ aMenu! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list' stamp: 'tomooda 12/3/2012 13:06'!
identifierSelectionIndex
	^ self identifierList indexOf: selectedIdentifier! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list' stamp: 'tomooda 11/16/2012 14:06'!
identifierSelectionIndex: anInteger
	self selectIdentifierNamed: (self identifierList at: anInteger ifAbsent: [])! !

!SIVDMInspectiveBrowser methodsFor: 'switches' stamp: 'tomooda 11/16/2012 11:41'!
indicateSource
	self stateIndicated: false! !

!SIVDMInspectiveBrowser methodsFor: 'switches' stamp: 'tomooda 11/16/2012 11:41'!
indicateState
	self stateIndicated: true! !

!SIVDMInspectiveBrowser methodsFor: 'initialize-release' stamp: 'tomooda 2/26/2013 13:02'!
initialize
	super initialize.
	timeOfLastStateUpdate := Time millisecondClockValue! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 2/26/2013 13:35'!
moduleList
	^ vdmj
		ifNil: [Array new]
		ifNotNil: [vdmj moduleNames]! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 2/26/2013 13:04'!
moduleListMenu: aMenu 
	| referenceSubMenu |
	aMenu add: 'remove module' translated action: #removeModule.
	aMenu addLine.
	referenceSubMenu := MenuMorph new defaultTarget: self.
	self referringModules
		do: [:name | referenceSubMenu
				add: name
				selector: #selectModuleNamed:
				argument: name].
	referenceSubMenu hasItems
		ifTrue: [aMenu add: 'referring modules...' translated subMenu: referenceSubMenu].
	^ aMenu! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 2/26/2013 13:13'!
moduleNamesFrom: aString 
	| modules whiteSpec |
	modules := Array new writeStream.
	whiteSpec := (vdmj whiten: aString) readStream.
	[vdmj
		skip: whiteSpec
		upToKeyword: 'module'
		ifNone: [^ modules contents].
	vdmj
		skipSeparators: whiteSpec
		ifAtEnd: [^ modules contents].
	modules
		nextPut: (vdmj readNonSeparators: whiteSpec)] repeat! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 2/26/2013 13:13'!
modulePositionsFrom: aString 
	| modulePositions whiteSpec mark |
	modulePositions := Array new writeStream.
	whiteSpec := (vdmj whiten: aString) readStream.
	mark := nil.
	[| newMark |
	[(whiteSpec skipTo: $m)
		ifFalse: [mark
				ifNil: [modulePositions nextPut: vdmj globalModuleName
							-> (1 to: aString size)]
				ifNotNil: [modulePositions nextPut: mark key
							-> (mark value to: whiteSpec position)].
			^ modulePositions contents].
	(whiteSpec next: 'odule' size)
		= 'odule'] whileFalse.
	newMark := whiteSpec position - 'module' size + 1.
	mark
		ifNotNil: [modulePositions nextPut: mark key
					-> (mark value to: newMark - 1)].
	whiteSpec skipSeparators.
	String new writeStream
		in: [:name | 
			[whiteSpec atEnd not
				and: [whiteSpec peek isSeparator not]]
				whileTrue: [name nextPut: whiteSpec next].
			mark := name contents -> newMark]] repeat! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 11/16/2012 11:25'!
moduleSelectionIndex
	^ self moduleList indexOf: self selectedModule! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 11/16/2012 11:25'!
moduleSelectionIndex: anInteger 
	^ self
		selectModuleNamed: (self moduleList
				at: anInteger
				ifAbsent: [])! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 11/20/2012 10:44'!
printItIn: aPluggableTextMorph 
	aPluggableTextMorph
		handleEdit: [| editor result selectionInterval |
			editor := aPluggableTextMorph textMorph editor.
			editor
				lineSelectAndEmptyCheck: [^ nil].
			result := self evaluate: editor selectionAsStream upToEnd.
			selectionInterval := editor selectionInterval.
			aPluggableTextMorph textMorph installEditorToReplace: editor.
			aPluggableTextMorph textMorph
				handleEdit: [editor afterSelectionInsertAndSelect: result].
			selectionInterval := editor selectionInterval.
			aPluggableTextMorph textMorph editor selectFrom: selectionInterval first to: selectionInterval last.
			aPluggableTextMorph scrollSelectionIntoView]! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 2/26/2013 13:04'!
referringModules
	| parser |
	self selectedModule isNil
		ifTrue: [^ Array new].
	parser := 'from' asParser , #space asParser plus , self selectedModule asParser , #space asParser.
	^ ((self modulePositionsFrom: vdmj specification)
		select: [:assoc | assoc key ~= self selectedModule
				and: [(parser
						matchesIn: (vdmj specification copyFrom: assoc value first to: assoc value last)) notEmpty]])
		collect: [:assoc | assoc key]! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list menu' stamp: 'tomooda 12/4/2012 10:48'!
removeIdentifier
	(self removeIdentifier: self selectedIdentifier module: self selectedModule)
		ifNotNil: [self changed: #identifierList.
			self selectIdentifierNamed: nil.
			self changed: #identifierSelectionIndex.
			vdm
				ifNotNil: [vdm
						swapSpecification: specification specificationString
						ifError: [:msg | 
							SINotifier notify: msg.
							^ false].
					self updateState].
			^ true].
	^ false! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list menu' stamp: 'tomooda 12/5/2012 07:41'!
removeIdentifier: identifier module: moduleName 
	specification
		ifNotNil: [(specification
				moduleAt: moduleName
				ifAbsent: [])
				ifNotNil: [:module | identifier
						ifNotNil: [self selectedSection = 'types'
								ifTrue: [^ (module
										removeType: identifier
										ifError: [:msg | 
											SINotifier notify: msg.
											nil]) notNil].
							self selectedSection = 'values'
								ifTrue: [^ (module
										removeValue: identifier
										ifError: [:msg | 
											SINotifier notify: msg.
											nil]) notNil].
							self selectedSection = 'functions'
								ifTrue: [^ (module
										removeFunction: identifier
										ifError: [:msg | 
											SINotifier notify: msg.
											nil]) notNil].
							self selectedSection = 'state'
								ifTrue: [^ (module
										removeStateIfError: [:msg | 
											SINotifier notify: msg.
											nil]) notNil].
							self selectedSection = 'operations'
								ifTrue: [^ (module
										removeOperation: identifier
										ifError: [:msg | 
											SINotifier notify: msg.
											nil]) notNil]]]].
	^ false! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 2/26/2013 13:04'!
removeModule
	self selectedModule
		in: [:moduleName | 
			| position |
			moduleName
				ifNil: [^ false].
			position := ((self modulePositionsFrom: vdmj specification)
						detect: [:assoc | assoc key = moduleName]
						ifNone: [^ false]) value.
			vdmj
				swapSpecification: (vdmj specification copyFrom: 1 to: position first - 1)
						, (vdmj specification copyFrom: position last + 1 to: vdmj specification size)
				ifError: [:msg | SINotifier notify: msg]]! !

!SIVDMInspectiveBrowser methodsFor: 'section list' stamp: 'tomooda 12/5/2012 08:18'!
sectionList
	^ self moduleSelectionIndex > 0 ifTrue: [#('exports' 'imports' 'types' 'values' 'functions' 'state' 'operations') copy] ifFalse: [Array new]! !

!SIVDMInspectiveBrowser methodsFor: 'section list' stamp: 'tomooda 11/16/2012 14:02'!
sectionSelectionIndex
	^ self sectionList indexOf: self selectedSection! !

!SIVDMInspectiveBrowser methodsFor: 'section list' stamp: 'tomooda 11/16/2012 14:03'!
sectionSelectionIndex: anInteger
	self selectSectionNamed: (self sectionList at: anInteger ifAbsent: [])! !

!SIVDMInspectiveBrowser methodsFor: 'field list' stamp: 'tomooda 11/16/2012 14:44'!
selectFieldNamed: aString 
	selectedField := (self fieldList includes: aString)
				ifTrue: [aString].
	self updateContents.
	self changed: #fieldSelectionIndex.
	self changed: #contents! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list' stamp: 'tomooda 12/3/2012 13:07'!
selectIdentifierNamed: aString
	selectedIdentifier := aString.
	self changed: #identifierSelectionIndex.
	self changed: #code! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 2/26/2013 13:03'!
selectModuleNamed: aString 
	selectedModule := (self moduleList includes: aString)
				ifTrue: [aString].
	self updateState.
	selectedField := nil.
	self updateContents.
	self changed: #fieldList.
	self changed: #fieldSelectionIndex.
	self changed: #code.
	self changed: #moduleSelectionIndex! !

!SIVDMInspectiveBrowser methodsFor: 'section list' stamp: 'tomooda 12/3/2012 13:12'!
selectSectionNamed: aString
	selectedSection := aString.
	selectedIdentifier := nil.
	self changed: #sectionSelectionIndex.
	self changed: #identifierList.
	self changed: #selectedIdentifierIndex.
	self changed: #code! !

!SIVDMInspectiveBrowser methodsFor: 'field list' stamp: 'tomooda 11/16/2012 13:48'!
selectedField
	^ selectedField! !

!SIVDMInspectiveBrowser methodsFor: 'identifier list' stamp: 'tomooda 11/16/2012 14:05'!
selectedIdentifier
	^ selectedIdentifier! !

!SIVDMInspectiveBrowser methodsFor: 'module list' stamp: 'tomooda 11/16/2012 11:26'!
selectedModule
	^ selectedModule! !

!SIVDMInspectiveBrowser methodsFor: 'section list' stamp: 'tomooda 11/16/2012 14:02'!
selectedSection
	^ selectedSection! !

!SIVDMInspectiveBrowser methodsFor: 'switches' stamp: 'tomooda 11/16/2012 11:36'!
sourceIndicated
	^ stateIndicated ~= true! !

!SIVDMInspectiveBrowser methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 09:26'!
specification
	^ specification! !

!SIVDMInspectiveBrowser methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 09:26'!
specification: aVDMSpecification
	specification := aVDMSpecification! !

!SIVDMInspectiveBrowser methodsFor: 'switches' stamp: 'tomooda 11/16/2012 11:35'!
stateIndicated
	^ stateIndicated = true! !

!SIVDMInspectiveBrowser methodsFor: 'switches' stamp: 'tomooda 2/26/2013 13:43'!
stateIndicated: aBoolean 
	stateIndicated := aBoolean = true.
	stateIndicated
		ifTrue: [self updateState.
			inspectorPanel owner addMorphFront: inspectorPanel]
		ifFalse: [codePane owner addMorphFront: codePane].
	self changed! !

!SIVDMInspectiveBrowser methodsFor: 'stepping' stamp: 'tomooda 11/16/2012 15:18'!
stepAt: millisecondClockValue in: aWindow 
	(Preferences smartUpdating
			and: [millisecondClockValue - self timeOfLastStateUpdate > (1000.0 / self updateFrequency)])
		ifTrue: [self stateIndicated
				ifTrue: [self updateState.
					self updateContents].
			timeOfLastStateUpdate := millisecondClockValue]! !

!SIVDMInspectiveBrowser methodsFor: 'stepping' stamp: 'tomooda 11/19/2012 17:51'!
stepTime
	^ 10! !

!SIVDMInspectiveBrowser methodsFor: 'stepping' stamp: 'tomooda 11/19/2012 17:52'!
stepTimeIn: aMorph
	^ self stepTime! !

!SIVDMInspectiveBrowser methodsFor: 'stepping' stamp: 'tomooda 11/16/2012 15:15'!
timeOfLastStateUpdate
	^ timeOfLastStateUpdate
		ifNil: [Time millisecondClockValue]! !

!SIVDMInspectiveBrowser methodsFor: 'value pane' stamp: 'tomooda 2/26/2013 13:06'!
updateContents
	| newContents |
	newContents := (selectedField
				ifNotNil: [(state
						detect: [:fieldAndValue | fieldAndValue key = selectedField]
						ifNone: [])
						ifNotNil: [:fieldAndValue | fieldAndValue value]])
				ifNil: [selectedModule
						ifNil: [String new]
						ifNotNil: [:module | 
							| content |
							content := String new writeStream.
							content nextPut: ${.
							state
								do: [:nameAndValue | content nextPut: $';
										 nextPutAll: nameAndValue key;
										 nextPut: $';
										 nextPutAll: ' -> ';
										 nextPut: $';
										 nextPutAll: nameAndValue value;
										 nextPut: $']
								separatedBy: [content space; nextPut: $.; cr].
							content nextPut: $}.
							content contents]].
	newContents ~= contents
		ifTrue: [contents := newContents.
			self changed: #contents]! !

!SIVDMInspectiveBrowser methodsFor: 'stepping' stamp: 'tomooda 11/19/2012 17:47'!
updateFrequency
	^ 10.0! !

!SIVDMInspectiveBrowser methodsFor: 'private' stamp: 'tomooda 2/26/2013 13:13'!
updateState
	| newState |
	newState := (vdmj
				ifNotNil: [selectedModule
						ifNotNil: [vdmj stateAt: selectedModule]])
				ifNil: [Array new].
	newState ~= state
		ifTrue: [state := newState.
			self updateContents.
			self changed: #moduleList.
			self changed: #moduleSelectionIndex.
			self changed: #fieldList.
			self changed: #fieldSelectionIndex.
			self changed: #contents.
			self changed: #state]! !

!SIVDMInspectiveBrowser methodsFor: 'value pane' stamp: 'tomooda 2/26/2013 13:06'!
valuePaneAccept
	^ self accept: valuePane text asString! !

!SIVDMInspectiveBrowser methodsFor: 'value pane' stamp: 'tomooda 2/26/2013 13:06'!
valuePaneDoIt
	self doItIn: valuePane! !

!SIVDMInspectiveBrowser methodsFor: 'value pane' stamp: 'tomooda 2/26/2013 13:06'!
valuePaneMenu: aMenu shifted: aBoolean
	aMenu addList: (aBoolean = true 
		ifTrue:[{
			#-.
			{'save contents to file...' translated.		#saveContentsInFile}.
			{'send contents to printer' translated.		#sendContentsToPrinter}.
			{'printer setup' translated.				#printerSetup}.
			#-.
		}.]
		ifFalse:[{
			{'set font... (k)' translated.				#offerFontMenu}.
			{'set style... (K)' translated.				#changeStyle}.
			{'set alignment... (u)' translated.		#chooseAlignment}.
			#-.
			{'make project link (P)' translated.	#makeProjectLink}.
			#-.
			{'find...(f)' translated.					#find}.
			{'find again (g)' translated.				#findAgain}.
			{'set search string (h)' translated.		#setSearchString}.
			#-.
			{'do again (j)' translated.				#again}.
			{'undo (z)' translated.					#undo}.
			#-.
			{'copy (c)' translated.					#copySelection}.
			{'cut (x)' translated.						#cut}.
			{'paste (v)' translated.					#paste}.
			{'paste...' translated.					#pasteRecent}.
			#-.
			{'do it (d)' translated.		#valuePaneDoIt}.			     
			{'print it (p)' translated.		#valuePanePrintIt}.
			#-.
			{'accept (s)' translated.					#valuePaneAccept}.
			{'cancel (l)' translated.					#valuePaneCancel}.
			#-.
			{'copy html' translated.					#copyHtml}.
			#-.
			{'more...' translated.					#shiftedTextPaneMenuRequest}.
		}]).
	^aMenu! !

!SIVDMInspectiveBrowser methodsFor: 'value pane' stamp: 'tomooda 2/26/2013 13:07'!
valuePanePrintIt
	^ self printItIn: valuePane! !

!SIVDMInspectiveBrowser methodsFor: 'accessing' stamp: 'tomooda 11/16/2012 11:47'!
vdm
	^vdm! !

!SIVDMInspectiveBrowser methodsFor: 'accessing' stamp: 'tomooda 11/16/2012 11:48'!
vdm: anSIVDM
	vdm := anSIVDM! !

!SIVDMInspectiveBrowser methodsFor: 'accessing' stamp: 'tomooda 2/26/2013 13:02'!
vdmj
	^vdmj! !

!SIVDMInspectiveBrowser methodsFor: 'accessing' stamp: 'tomooda 2/26/2013 13:02'!
vdmj: anSIVDM
	vdmj := anSIVDM! !

!SIVDMInspectiveBrowser methodsFor: 'stepping' stamp: 'tomooda 11/16/2012 15:14'!
wantsSteps
	^ true! !

SIExpressionPrinter subclass: #SIVDMExpressionPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Process'!

!SIVDMExpressionPrinter methodsFor: 'defaults' stamp: 'tomooda 2/14/2013 11:46'!
defaultClassMap
	| mappings |
	mappings := Array new writeStream.
	mappings nextPut: #UndefinedObject -> #printNil:on:.
	mappings nextPut: #SIAlgebraicData -> #printData:on:.
	mappings nextPut: #Set -> #printSet:on:.
	mappings nextPut: #Dictionary -> #printMap:on:.
	mappings nextPut: #Symbol -> #printQuote:on:.
	mappings nextPut: #SIVDMTokenValue -> #printToken:on:.
	mappings nextPutAll: super defaultClassMap.
	^ mappings contents! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 7/25/2012 15:11'!
printBoolean: anObject on: aStream 
	aStream
		nextPutAll: (anObject == true
				ifTrue: ['true']
				ifFalse: ['false'])! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 10/18/2012 11:16'!
printData: anSIAlgebraicDataType on: aStream 
	^ self
		printFunc: anSIAlgebraicDataType constructor
		arguments: anSIAlgebraicDataType args
		on: aStream! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 7/27/2012 17:39'!
printFunc: aString arguments: anArray on: aStream 
	aStream nextPutAll: aString.
	aStream nextPut: $(.
	anArray
		do: [:object | 
			self print: object on: aStream]
		separatedBy: [aStream nextPut: $,].
	aStream nextPut: $)! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'tomooda 2/14/2013 11:47'!
printNil: anObject on: aStream 
	aStream nextPutAll: 'nil'! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 7/25/2012 17:29'!
printQuote: aString on: aStream 
	aStream nextPut: $<.
	aStream nextPutAll: aString.
	aStream nextPut: $>! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 9/13/2012 12:41'!
printSet: anObject on: aStream 
	aStream nextPut: ${.
	anObject
		do: [:member | self print: member on: aStream]
		separatedBy: [aStream nextPut: $,].
	aStream nextPut: $}! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 7/25/2012 15:31'!
printToken: aVDMTokenValue on: aStream 
	aStream nextPutAll: 'mk_token('.
	self print: aVDMTokenValue value on: aStream.
	aStream nextPut: $)! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 7/25/2012 15:20'!
printTuple: anObject on: aStream 
	aStream nextPutAll: 'mk_('.
	anObject
		do: [:member | self print: member on: aStream]
		separatedBy: [aStream nextPut: $,].
	aStream nextPut: $)! !

!SIVDMExpressionPrinter methodsFor: 'printing' stamp: 'TO 9/20/2012 11:30'!
printUnit: anObject on: aStream 
	aStream nextPutAll: '()'! !

SIExpressionData subclass: #SIVDMConstant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Data'!

!SIVDMConstant methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:44'!
value
	^ environment evaluate: expression! !

!SIVDMConstant methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:44'!
value: anObject
	^ self shouldNotImplement! !

!SIVDMConstant methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:45'!
value: arg1 value: arg2
	^ self shouldNotImplement! !

!SIVDMConstant methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:45'!
value: arg1 value: arg2 value: arg3
	^ self shouldNotImplement! !

!SIVDMConstant methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:45'!
valueWithArguments: anArray
	^ self shouldNotImplement! !

SIExpressionData subclass: #SIVDMFunction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Data'!

!SIVDMFunction methodsFor: 'evaluating' stamp: 'TO 9/20/2012 11:32'!
value
	^ environment evaluate: expression, '()'! !

!SIVDMFunction methodsFor: 'evaluating' stamp: 'TO 9/20/2012 11:35'!
value: argument 
	^ self
		valueWithArguments: (Array with: argument)! !

!SIVDMFunction methodsFor: 'evaluating' stamp: 'TO 9/20/2012 11:35'!
value: argument1 value: argument2 
	^ self
		valueWithArguments: (Array with: argument1 with: argument2)! !

!SIVDMFunction methodsFor: 'evaluating' stamp: 'TO 9/20/2012 11:36'!
value: argument1 value: argument2 value: argument3 
	^ self
		valueWithArguments: (Array
				with: argument1
				with: argument2
				with: argument3)! !

!SIVDMFunction methodsFor: 'evaluating' stamp: 'TO 9/20/2012 11:35'!
valueWithArguments: anArray 
	| call |
	call := String new writeStream.
	environment printer
		printFunc: expression
		arguments: anArray
		on: call.
	^ environment evaluate: call contents! !

SIVDMFunction subclass: #SIVDMOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Data'!

SIExpressionData subclass: #SIVDMVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Data'!

!SIVDMVariable methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:52'!
value
	^ environment evaluate: expression! !

!SIVDMVariable methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:52'!
value: anObject
	^ self shouldNotImplement! !

!SIVDMVariable methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:52'!
value: arg1 value: arg2
	^ self shouldNotImplement! !

!SIVDMVariable methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:52'!
value: arg1 value: arg2 value: arg3
	^ self shouldNotImplement! !

!SIVDMVariable methodsFor: 'evaluating' stamp: 'tomooda 11/13/2012 11:52'!
valueWithArguments: anArray
	^ self shouldNotImplement! !

SIAlgebraicData variableSubclass: #SIVDMRecordValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Process'!

!SIVDMRecordValue class methodsFor: 'instance creation' stamp: 'TO 7/25/2012 10:44'!
of: aString withAll: anArray 
	| record |
	record := self new: anArray size.
	record setConstructor: aString.
	1
		to: anArray size
		do: [:index | record
				at: index
				put: (anArray at: index)].
	^ record! !

!SIVDMRecordValue methodsFor: 'private' stamp: 'TO 7/27/2012 10:55'!
first
	^ self at: 1! !

!SIVDMRecordValue methodsFor: 'private' stamp: 'TO 7/27/2012 10:56'!
last
	^ self at: self size! !

!SIVDMRecordValue methodsFor: 'printing' stamp: 'TO 7/25/2012 14:41'!
printOn: aStream 
	aStream nextPutAll: constructor.
	aStream nextPut: $(.
	self size > 0 ifTrue: [(self at: 1)
		printOn: aStream].
	2
		to: self size
		do: [:index | 
			aStream nextPutAll: ', '.
			(self at: index)
				printOn: aStream].
	aStream nextPut: $)! !

!SIVDMRecordValue methodsFor: 'private' stamp: 'TO 7/27/2012 10:55'!
second
	^ self at: 2! !

!SIVDMRecordValue methodsFor: 'printing' stamp: 'TO 7/25/2012 10:44'!
storeOn: aStream
	aStream nextPutAll: self class name.
	aStream nextPutAll: ' of: '.
	constructor storeOn: aStream.
	aStream nextPutAll: ' withAll: {'.
	(self at: 1) storeOn: aStream .
	2 to: self size do: [ :index |
		aStream nextPutAll: ' . '.
		(self at: index) storeOn: aStream].
	aStream nextPut: $}! !

!SIVDMRecordValue methodsFor: 'private' stamp: 'TO 7/27/2012 10:56'!
third
	^ self at: 3! !

!String methodsFor: '*SomethingIt-VDM-converting' stamp: 'tomooda 11/13/2012 11:53'!
asVDMConstantIn: anSIVDM
	^SIVDMConstant with: self in: anSIVDM! !

!String methodsFor: '*SomethingIt-VDM-converting' stamp: 'TO 9/20/2012 10:35'!
asVDMExpressionIn: anSIVDM
	^SIExpressionData with: self in: anSIVDM! !

!String methodsFor: '*SomethingIt-VDM-converting' stamp: 'TO 9/20/2012 11:37'!
asVDMFunctionIn: anSIVDM
	^SIVDMFunction with: self in: anSIVDM! !

!String methodsFor: '*SomethingIt-VDM-converting' stamp: 'TO 9/20/2012 11:37'!
asVDMOperationIn: anSIVDM 
	^ SIVDMOperation with: self in: anSIVDM! !

!String methodsFor: '*SomethingIt-VDM-converting' stamp: 'tomooda 11/13/2012 12:00'!
asVDMVariableIn: anSIVDM
	^SIVDMVariable with: self in: anSIVDM! !

SIExternalInterpreter subclass: #SIVDM
	instanceVariableNames: 'stateVariables modulePositions originalSpecification generatedSpecification snapshot tmpFilename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Process'!

!SIVDM class methodsFor: 'examples' stamp: 'tomooda 11/14/2012 09:50'!
exampleClickableGravityByFunction
	"SIVDM exampleClickableGravityByFunction"
	| gravity form morph window colors lock  |
	form := Form extent: 500 @ 500 depth: 32.
	morph := form asMorph.
	window := morph openInWindowLabeled: 'Gravity by VDM functions'.
	colors := {Color red . Color green . Color blue} asOrderedCollection.
	lock := Semaphore forMutualExclusion.
	[| vdm points velocities add |
	vdm := SIVDM specification: '
module Point
exports all
definitions
types
  Point :: x : real
           y : real;
functions
  plus : Point * Point -> Point
  plus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1+x2, y1+y2);

  minus : Point * Point -> Point
  minus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1-x2, y1-y2);

  neg : Point -> Point
  neg(mk_Point(x,y)) == mk_Point(-x, -y);

  scale : real * Point -> Point
  scale(a, mk_Point(x,y)) == mk_Point(a*x, a*y);

  dist2 : Point -> real
  dist2(mk_Point(x,y)) == x * x + y * y;

  dist : Point -> real
  dist(p) == dist2(p) ** 0.5;
end Point

module Gravity
imports from Point
    types
        Point renamed Point;
    functions
        plus renamed plus;
        minus renamed minus;
        neg renamed neg;
        scale renamed scale;
        dist2 renamed dist2;
        dist renamed dist
exports all
definitions
values
  gmn = 50.0;
functions
  total : seq of Point -> Point
  total(ps) == cases ps :
    [] -> mk_Point(0.0,0.0),
    others -> plus(hd ps, total(tl ps))
    end;

  gravity1 : Point * Point -> Point
  gravity1(p1, p2) ==
    let
      v = minus(p2, p1),
      d2 = dist2(v)
    in
      if d2 < 400.0
      then mk_Point(0.0, 0.0)
      else scale(gmn/(d2**1.5), v);

  gravity : seq of Point -> seq of Point
  gravity(ps) == [total([gravity1(ps(i1), ps(i2)) | i2 in set {1,...,len ps}]) | i1 in set {1,...,len ps}];
end Gravity
'.
	vdm printer
		addClassMap: #Point
		to: [:point :vdmStream | 
			vdmStream nextPutAll: 'mk_Point`Point('.
			point x printOn: vdmStream.
			vdmStream nextPut: $,.
			point y printOn: vdmStream.
			vdmStream nextPut: $)].
	vdm parser
		addAlgebraicDataMap: 'mk_Point'
		to: [:constructor :args | args first @ args second].
	gravity := 'Gravity`gravity' asVDMFunctionIn: vdm.
	points := {100.0 @ 250.0. 250.0 @ 250.0. 250.0 @ 350.0} asOrderedCollection.
	velocities := {0.0 @ 0.0. 0.0 @ 0.0. 0.0 @ 0.0} asOrderedCollection.
	add := [:array1 :array2 | 1
				to: array1 size
				do: [:index | array1 at: index put: (array1 at: index)
							+ (array2 at: index)]].
	morph
		on: #click
		send: #value
		to: [lock critical: [points add: ActiveHand position - morph position.
			velocities add: 0 @ 0.
			colors add: Color black]].
	2000
		timesRepeat: [
			| canvas |
			canvas := form getCanvas.
			canvas fillColor: Color white.
			lock critical: [| forces |
			forces := gravity value: points.
			add value: velocities value: forces.
			add value: points value: velocities.
			points
				with: colors
				do: [:point :color | canvas
						fillOval: (point - 4 extent: 8 @ 8)
						color: color]].
			canvas flush.
			morph changed.
			(Delay forMilliseconds: 20) wait].
	morph delete.
	window delete]
		forkAt: Processor userBackgroundPriority! !

!SIVDM class methodsFor: 'examples' stamp: 'tomooda 11/14/2012 13:18'!
exampleFizzbuzz
	"SIVDM exampleFizzbuzz"

	| fizzbuzzSpec count sayFizz sayBuzz sayFizzBuzz sayNumber window font fizzButton buzzButton fizzbuzzButton numberButton |
	fizzbuzzSpec := SIVDM
		specification:
			'
/*
The FizzBuzz game counts numbers up,
saying Fizz when the number is divisible by 3,
Buzz when the number is divisible by 5,
FizzBuzz when the number is divisible by 15,
otherwise the number.
*/
state counter of
  count : int
init
  -- the game starts with "1".
  s == s = mk_counter(1)
end

functions
  -- say fizz when divisible by 3
  fizz : int -> seq of char
  fizz(x) == if x mod 3 = 0 then "fizz" else "";

  -- say buzz when divisible by 5
  buzz : int -> seq of char
  buzz(x) == if x mod 5 = 0 then "buzz" else "";

  -- say fizzbuzz when divisible by 15, otherwise the number.
  fizzbuzz : int -> token
  fizzbuzz(x) == let fb = fizz(x) ^ buzz(x) in if fb = "" then mk_token(x) else mk_token(fb);

operations
  -- A player says "fizz". Answers whether or not it is legitimate call.
  sayFizz : () ==> bool
  sayFizz() == if fizzbuzz(count) = mk_token("fizz") then (count := count + 1; return true) else return false; 
  -- A player says "buzz". Answers whether or not it is legitimate call.
  sayBuzz : () ==> bool
  sayBuzz() == if fizzbuzz(count) = mk_token("buzz") then (count := count + 1; return true) else return false; 
  -- A player says "fizzbuzz". Answers whether or not it is legitimate call.
  sayFizzBuzz : () ==> bool
  sayFizzBuzz() == if fizzbuzz(count) = mk_token("fizzbuzz") then (count := count + 1; return true) else return false; 
  -- A player says a number. Answers whether or not it is legitimate call.
  sayNumber : () ==> bool
  sayNumber() == if fizzbuzz(count) = mk_token(count) then (count := count + 1; return true) else return false; 

'.
	sayFizz := 'sayFizz' asVDMOperationIn: fizzbuzzSpec.
	sayBuzz := 'sayBuzz' asVDMOperationIn: fizzbuzzSpec.
	sayFizzBuzz := 'sayFizzBuzz' asVDMOperationIn: fizzbuzzSpec.
	sayNumber := 'sayNumber' asVDMOperationIn: fizzbuzzSpec.
	count := 'count' asVDMVariableIn: fizzbuzzSpec.
	font := TextStyle default fontOfSize: 36.
	window := Morph new.
	window extent: 150 @ 150.
	window color: Color white.
	window changeTableLayout.
	window listDirection: #topToBottom.
	window vResizing: #shrinkWrap.
	numberButton := SimpleButtonMorph newWithLabel: count value printString.
	numberButton label: numberButton label font: font.
	numberButton
		target: [ 
			sayNumber value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	numberButton actionSelector: #value.
	numberButton hResizing: #spaceFill.
	window addMorphBack: numberButton.
	fizzButton := SimpleButtonMorph newWithLabel: 'Fizz'.
	fizzButton label: fizzButton label font: font.
	fizzButton
		target: [ 
			sayFizz value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	fizzButton actionSelector: #value.
	fizzButton hResizing: #spaceFill.
	window addMorphBack: fizzButton.
	buzzButton := SimpleButtonMorph newWithLabel: 'Buzz'.
	buzzButton label: buzzButton label font: font.
	buzzButton
		target: [ 
			sayBuzz value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	buzzButton actionSelector: #value.
	buzzButton hResizing: #spaceFill.
	window addMorphBack: buzzButton.
	fizzbuzzButton := SimpleButtonMorph newWithLabel: 'FizzBuzz'.
	fizzbuzzButton label: fizzbuzzButton label font: font.
	fizzbuzzButton
		target: [ 
			sayFizzBuzz value
				ifTrue: [ numberButton label: count value printString font: font ]
				ifFalse: [ SINotifier notify: 'WRONG ANSWER!!' ] ].
	fizzbuzzButton actionSelector: #value.
	fizzbuzzButton hResizing: #spaceFill.
	window addMorphBack: fizzbuzzButton.
	window openInWindow.
	^ fizzbuzzSpec! !

!SIVDM class methodsFor: 'examples' stamp: 'tomooda 12/10/2012 06:41'!
exampleGravityByFunction
	"SIVDM exampleGravityByFunction"
	| gravity form morph window colors |
	form := Form extent: 500 @ 500 depth: 32.
	morph := form asMorph.
	window := morph openInWindowLabeled: 'Gravity by VDM functions'.
	colors := (1 to: 3)
				collect: [:i | Color
						h: i * 360.0 / 10
						s: 1.0
						v: 1.0].
	[| vdm points velocities add |
	vdm := SIVDM specification: '
module Point
exports all
definitions
types
  Point :: x : real
           y : real;
functions
  plus : Point * Point -> Point
  plus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1+x2, y1+y2);

  minus : Point * Point -> Point
  minus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1-x2, y1-y2);

  neg : Point -> Point
  neg(mk_Point(x,y)) == mk_Point(-x, -y);

  scale : real * Point -> Point
  scale(a, mk_Point(x,y)) == mk_Point(a*x, a*y);

  dist2 : Point -> real
  dist2(mk_Point(x,y)) == x * x + y * y;

  dist : Point -> real
  dist(p) == dist2(p) ** 0.5;
end Point

module Gravity
exports all
imports from Point
    types
        Point renamed Point;
    functions
        plus renamed plus;
        minus renamed minus;
        neg renamed neg;
        scale renamed scale;
        dist2 renamed dist2;
        dist renamed dist
definitions
values
  gmn = 50.0;
functions
  total : seq of Point -> Point
  total(ps) == cases ps :
    [] -> mk_Point(0.0,0.0),
    others -> plus(hd ps, total(tl ps))
    end;

  gravity1 : Point * Point -> Point
  gravity1(p1, p2) ==
    let
      v = minus(p2, p1),
      d2 = dist2(v)
    in
      if d2 < 400.0
      then mk_Point(0.0, 0.0)
      else scale(gmn/(d2**1.5), v);

  gravity : seq of Point -> seq of Point
  gravity(ps) == [total([gravity1(ps(i1), ps(i2)) | i2 in set {1,...,len ps}]) | i1 in set {1,...,len ps}];
end Gravity

module ParticlesSimulation
exports all
imports
    from Point
        types
            Point renamed Point;
        functions
            plus renamed plus,
    from Gravity
        functions
            gravity renamed gravity

definitions
state Particles of
  positions : seq of Point
  velocities : seq of Point
init
  particles == particles = mk_Particles([], [])
end
operations
  Step : () ==> ()
  Step() == (
    velocities := (let forces = gravity(positions) in [plus(velocities(i), forces(i)) | i in set {1,...,len forces}]);
    positions := [plus(positions(i), velocities(i)) | i in set {1,...,len velocities}]);
  GetPositions : () ==> seq of Point
  GetPositions() == return positions;
  Init : seq of Point * seq of Point ==> ()
  Init(ps, vs) == (positions := ps; velocities := vs);
end ParticlesSimulation
'.
	vdm printer
		addClassMap: #Point
		to: [:point :vdmStream | 
			vdmStream nextPutAll: 'mk_Point`Point('.
			point x printOn: vdmStream.
			vdmStream nextPut: $,.
			point y printOn: vdmStream.
			vdmStream nextPut: $)].
	vdm parser
		addAlgebraicDataMap: 'mk_Point'
		to: [:constructor :args | args first @ args second].
	gravity := 'Gravity`gravity' asVDMFunctionIn: vdm.
	points := {100.0 @ 250.0. 250.0 @ 250.0. 250.0 @ 350.0} asOrderedCollection.
	velocities := {0.0 @ 0.0. 0.0 @ 0.0. 0.0 @ 0.0} asOrderedCollection.
	add := [:array1 :array2 | 1
				to: array1 size
				do: [:index | array1 at: index put: (array1 at: index)
							+ (array2 at: index)]].
	2000
		timesRepeat: [| forces canvas |
			forces := gravity value: points.
			add value: velocities value: forces.
			add value: points value: velocities.
			canvas := form getCanvas.
			canvas fillColor: Color white.
			points
				with: colors
				do: [:point :color | canvas
						fillOval: (point - 4 extent: 8 @ 8)
						color: color].
			canvas flush.
			morph changed.
			(Delay forMilliseconds: 10) wait].
	morph delete.
	window delete]
		forkAt: Processor userBackgroundPriority! !

!SIVDM class methodsFor: 'examples' stamp: 'tomooda 11/14/2012 09:47'!
exampleGravityByOperation
	"SIVDM exampleGravityByOperation"
	| form morph window colors |
	form := Form extent: 500 @ 500 depth: 32.
	morph := form asMorph.
	window := morph openInWindowLabeled: 'Gravity by VDM operations'.
	colors := (1 to: 3)
				collect: [:i | Color
						h: i * 360.0 / 10
						s: 1.0
						v: 1.0].
	[| vdm step positions |
	vdm := SIVDM specification: '
module Point
exports all
definitions
types
  Point :: x : real
           y : real;
functions
  plus : Point * Point -> Point
  plus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1+x2, y1+y2);

  minus : Point * Point -> Point
  minus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1-x2, y1-y2);

  neg : Point -> Point
  neg(mk_Point(x,y)) == mk_Point(-x, -y);

  scale : real * Point -> Point
  scale(a, mk_Point(x,y)) == mk_Point(a*x, a*y);

  dist2 : Point -> real
  dist2(mk_Point(x,y)) == x * x + y * y;

  dist : Point -> real
  dist(p) == dist2(p) ** 0.5;
end Point

module Gravity
exports all
imports from Point
    types
        Point renamed Point;
    functions
        plus renamed plus;
        minus renamed minus;
        neg renamed neg;
        scale renamed scale;
        dist2 renamed dist2;
        dist renamed dist
definitions
values
  gmn = 50.0;
functions
  total : seq of Point -> Point
  total(ps) == cases ps :
    [] -> mk_Point(0.0,0.0),
    others -> plus(hd ps, total(tl ps))
    end;

  gravity1 : Point * Point -> Point
  gravity1(p1, p2) ==
    let
      v = minus(p2, p1),
      d2 = dist2(v)
    in
      if d2 < 400.0
      then mk_Point(0.0, 0.0)
      else scale(gmn/(d2**1.5), v);

  gravity : seq of Point -> seq of Point
  gravity(ps) == [total([gravity1(ps(i1), ps(i2)) | i2 in set {1,...,len ps}]) | i1 in set {1,...,len ps}];
end Gravity

module ParticlesSimulation
exports all
imports
    from Point
        types
            Point renamed Point;
        functions
            plus renamed plus,
    from Gravity
        functions
            gravity renamed gravity

definitions
state Particles of
  positions : seq of Point
  velocities : seq of Point
init
  particles == particles = mk_Particles([], [])
end
operations
  Step : () ==> ()
  Step() == (
    velocities := (let forces = gravity(positions) in [plus(velocities(i), forces(i)) | i in set {1,...,len forces}]);
    positions := [plus(positions(i), velocities(i)) | i in set {1,...,len velocities}]);
  GetPositions : () ==> seq of Point
  GetPositions() == return positions;
  Init : seq of Point * seq of Point ==> ()
  Init(ps, vs) == (positions := ps; velocities := vs);
end ParticlesSimulation
'.
	vdm printer
		addClassMap: #Point
		to: [:point :vdmStream | 
			vdmStream nextPutAll: 'mk_Point`Point('.
			point x printOn: vdmStream.
			vdmStream nextPut: $,.
			point y printOn: vdmStream.
			vdmStream nextPut: $)].
	vdm parser
		addAlgebraicDataMap: 'mk_Point'
		to: [:constructor :args | args first @ args second].
	('ParticlesSimulation`Init' asVDMOperationIn: vdm)
		value: {100.0 @ 250.0. 250.0 @ 250.0. 250.0 @ 350.0} asOrderedCollection
		value: {0.0 @ 0.0. 0.0 @ 0.0. 0.0 @ 0.0} asOrderedCollection.
	step := 'ParticlesSimulation`Step' asVDMOperationIn: vdm.
	positions := 'ParticlesSimulation`GetPositions' asVDMOperationIn: vdm.
	2000
		timesRepeat: [| canvas points |
			step value.
			points := positions value.
			canvas := form getCanvas.
			canvas fillColor: Color white.
			points
				with: colors
				do: [:point :color | canvas
						fillOval: (point - 4 extent: 8 @ 8)
						color: color].
			canvas flush.
			morph changed.
			(Delay forMilliseconds: 10) wait].
	morph delete.
	window delete]
		forkAt: Processor userBackgroundPriority! !

!SIVDM class methodsFor: 'examples' stamp: 'TO 9/20/2012 13:54'!
exampleHelloWorld
	"SIVDM exampleHelloWorld inspect"
	| vdm |
	vdm := SIVDM plain.
	^ [vdm evaluate: '"Hello World"']
		ensure: [vdm close]! !

!SIVDM class methodsFor: 'examples' stamp: 'TO 9/20/2012 13:54'!
exampleMath
	"SIVDM exampleMath inspect"
	| vdm |
	vdm := SIVDM plain.
	^ [vdm evaluate: '1 + 2 * 3']
		ensure: [vdm close]! !

!SIVDM class methodsFor: 'examples' stamp: 'tomooda 11/13/2012 11:55'!
exampleStack
	"SIVDM exampleStack inspect"
	| vdm push pop top stack |
	vdm := SIVDM specification: '
state Stack of
	stack : seq of int
init
	stack == stack = mk_Stack([])
end

operations
	Push : int ==> ()
	Push(c) == stack := [c] ^ stack;
	Pop : () ==> ()
	Pop() == stack := tl stack;
	Top : () ==> int
	Top() == return hd stack;
'.
	push := 'Push' asVDMOperationIn: vdm.
	pop := 'Pop' asVDMOperationIn: vdm.
	top := 'Top' asVDMOperationIn: vdm.
	stack := 'Stack' asVDMVariableIn: vdm.
	#(1 2 3 4 5 6 7 8 9 ) do: push. "performs push(1); push(2); ...; push(9)"
	3 timesRepeat: pop. "takes 9, 8 and 7 away."
	^ {top value. stack value}"will get 6 and mk_Stack([6,5,4,3,2,1])."! !

!SIVDM class methodsFor: 'examples' stamp: 'TO 9/20/2012 14:06'!
exampleTypeError
	"SIVDM exampleTypeError"
	SIVDM plain evaluate: '1+"a"'! !

!SIVDM class methodsFor: 'accessing' stamp: 'TO 10/19/2012 11:32'!
javaPath
	^ '/usr/bin/java'! !

!SIVDM class methodsFor: 'instance creation' stamp: 'tomooda 11/13/2012 13:44'!
load: aPathString 
	^ self plain load: aPathString;
		 yourself! !

!SIVDM class methodsFor: 'instance creation' stamp: 'TomohiroOda 7/20/2012 08:57'!
plain
	^ self new! !

!SIVDM class methodsFor: 'instance creation' stamp: 'tomooda 11/12/2012 13:38'!
specification: aString 
	^ self new
		specification: aString
		ifError: [:msg | 
			SINotifier notify: msg.
			^ nil];
		 yourself! !

!SIVDM class methodsFor: 'accessing' stamp: 'TO 10/19/2012 11:33'!
vdmjPath
	^ 'vdmj-2.0.1.jar'! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/9/2012 14:47'!
backSkipToNonSeparators: aStream 
	[aStream position > 0
		and: [aStream peek isSeparator]]
		whileTrue: [aStream skip: -1]! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 11/9/2012 14:35'!
backdoorGetter
	^ self backdoorPrefix , 'getState'! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 11/8/2012 15:05'!
backdoorPrefix
	^ '$SomethingIt$'! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 11/9/2012 14:36'!
backdoorSetter
	^ self backdoorPrefix , 'setState'! !

!SIVDM methodsFor: 'communicating' stamp: 'tomooda 2/14/2013 12:04'!
basicReadUpToPrompt
	| lfStream |
	lfStream := String new writeStream.
	[| chunk |
	chunk := reader upTo: $>.
	lfStream nextPutAll: chunk.
	chunk size <= 1
		or: [| last |
			last := chunk at: chunk size - 1.
			last = Character lf
				or: [last = $]]]] whileFalse.
	^ lfStream contents! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 11/9/2012 15:15'!
createBackdoorModuleForAll: anArrayOfModuleString on: aWriteStream 
	aWriteStream nextPut: Character cr;
		 nextPutAll: '-- the "', self globalModuleName, '" module is automatically generated by SomethingIt. --';
		 nextPut: Character cr.
	aWriteStream nextPutAll: 'module ';
		 nextPutAll: self globalModuleName;
		 nextPut: Character cr.
	aWriteStream nextPutAll: 'imports '.
	anArrayOfModuleString
		do: [:module | aWriteStream nextPutAll: 'from ';
				 nextPutAll: module;
				 space;
				 nextPutAll: 'all']
		separatedBy: [aWriteStream nextPutAll: ', '].
	aWriteStream nextPut: Character cr.
	aWriteStream nextPutAll: 'end ';
		 nextPutAll: self globalModuleName;
		 nextPut: Character cr! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 1/2/2013 21:20'!
createBackdoorsFrom: sourceString on: aWriteStream ifError: errorBlock 
	| sourceStream mappings |
	sourceStream := sourceString readStream.
	mappings := Array new writeStream.
	stateVariables := OrderedCollection new.
	modulePositions := Dictionary new.
	(self
			skip: sourceString readStream
			upToKeyword: 'module'
			ifNone: []) notNil
		ifTrue: [| modules lastPosition |
			modules := Array new writeStream.
			lastPosition := 0.
			self
				scan: (self whiten: sourceString) readStream
				modulesAndStatesAndPositionsDo: [:module :variables :exportsAssoc :currentPosition | 
					modules nextPut: module.
					stateVariables add: module -> variables copy.
					exportsAssoc value
						ifNotNil: [aWriteStream
								nextPutAll: (sourceStream next: exportsAssoc value key - lastPosition).
							sourceStream position: exportsAssoc value value.
							lastPosition := exportsAssoc value value.
							exportsAssoc key
								ifTrue: [aWriteStream nextPut: Character cr]
								ifFalse: [aWriteStream nextPutAll: 'exports '].
							aWriteStream nextPutAll: 'all -- this line is automatically generated by SomethingIt.';
								 nextPut: Character cr].
					aWriteStream
						nextPutAll: (sourceStream next: currentPosition - lastPosition).
					lastPosition := currentPosition.
					aWriteStream
						nextPutAll: (self
								createModuleBackdoorOperationFor: variables
								in: module
								ifError: [:msg | ^ errorBlock value: msg])]
				ifError: [:msg | ^ errorBlock value: msg].
			aWriteStream nextPutAll: sourceStream upToEnd.
			self createBackdoorModuleForAll: modules contents on: aWriteStream]
		ifFalse: [aWriteStream nextPutAll: sourceString.
			self
				scan: (self whiten: sourceString) readStream
				variablesDo: [:variables | 
					stateVariables add: self globalModuleName -> variables copy.
					aWriteStream nextPut: Character cr.
					aWriteStream
						nextPutAll: (self
								createFlatBackdoorOperationFor: variables
								ifError: [:msg | ^ errorBlock value: msg])]
				ifError: [:msg | ^ errorBlock value: msg]].
	^ mappings contents! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 11/14/2012 14:57'!
createFlatBackdoorOperationFor: variables ifError: errorBlock 
	| output |
	output := String new writeStream.
	output cr.
	output nextPutAll: '-- the two operations, "' , self backdoorSetter , '" and "' , self backdoorGetter , '", are automatically generated by SomethingIt. --';
		 cr.
	output nextPutAll: 'operations';
		 cr.
	output space; space; space; space.
	output nextPutAll: self backdoorSetter;
		 nextPutAll: ' : '.
	variables isEmpty
		ifTrue: [output nextPutAll: '()']
		ifFalse: [variables
				do: [:var | output nextPut: $?]
				separatedBy: [output nextPut: $*]].
	output nextPutAll: ' ==> ()';
		 cr.
	output space; space; space; space.
	output nextPutAll: self backdoorSetter;
		 nextPutAll: '('.
	variables
		do: [:variable | output nextPutAll: self backdoorPrefix;
				 nextPutAll: variable]
		separatedBy: [output nextPutAll: ', '].
	output nextPutAll: ') == '.
	variables isEmpty
		ifTrue: [output nextPutAll: 'skip']
		ifFalse: [output nextPutAll: 'atomic('.
			variables
				do: [:variable | output nextPutAll: variable;
						 nextPutAll: ' := ';
						 nextPutAll: self backdoorPrefix;
						 nextPutAll: variable]
				separatedBy: [output nextPutAll: ' ; '].
			output nextPutAll: ')'].
	output nextPutAll: ';';
		 cr.
	output space; space; space; space.
	output nextPutAll: self backdoorGetter;
		 nextPutAll: ' : () ==> '.
	variables isEmpty
		ifTrue: [output nextPutAll: '()']
		ifFalse: [variables
				do: [:var | output nextPut: $?]
				separatedBy: [output nextPut: $*]].
	output cr.
	output space; space; space; space.
	output nextPutAll: self backdoorGetter;
		 nextPutAll: '() == '.
	variables isEmpty
		ifTrue: [output nextPutAll: 'skip']
		ifFalse: [output nextPutAll: 'return '.
			variables size > 1
				ifTrue: [output nextPutAll: 'mk_('].
			variables
				do: [:variable | output nextPutAll: variable]
				separatedBy: [output nextPutAll: ','].
			variables size > 1
				ifTrue: [output nextPutAll: ')']].
	output nextPutAll: ';';
		 cr.
	^ output contents! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 11/14/2012 14:57'!
createModuleBackdoorOperationFor: variables in: moduleString ifError: errorBlock 
	| output |
	output := String new writeStream.
	output cr.
	output nextPutAll: '-- the two operations, "' , self backdoorSetter , '" and "' , self backdoorGetter , '", are automatically generated and exported by SomethingIt. --';
		 cr.
	output nextPutAll: 'operations';
		 cr.
	output space; space; space; space.
	output nextPutAll: self backdoorSetter;
		 nextPutAll: ' : '.
	variables isEmpty
		ifTrue: [output nextPutAll: '()']
		ifFalse: [variables
				do: [:var | output nextPut: $?]
				separatedBy: [output nextPut: $*]].
	output nextPutAll: ' ==> ()';
		 cr.
	output space; space; space; space.
	output nextPutAll: self backdoorSetter;
		 nextPutAll: '('.
	variables
		do: [:variable | output nextPutAll: self backdoorPrefix;
				 nextPutAll: variable]
		separatedBy: [output nextPutAll: ', '].
	output nextPutAll: ') == '.
	variables isEmpty
		ifTrue: [output nextPutAll: 'skip']
		ifFalse: [output nextPutAll: 'atomic('.
			variables
				do: [:variable | output nextPutAll: variable;
						 nextPutAll: ' := ';
						 nextPutAll: self backdoorPrefix;
						 nextPutAll: variable]
				separatedBy: [output nextPutAll: ' ; '].
			output nextPutAll: ')'].
	output nextPutAll: ';';
		 cr.
	output space; space; space; space.
	output nextPutAll: self backdoorGetter;
		 nextPutAll: ' : () ==> '.
	variables isEmpty
		ifTrue: [output nextPutAll: '()']
		ifFalse: [variables
				do: [:var | output nextPut: $?]
				separatedBy: [output nextPut: $*]].
	output cr.
	output space; space; space; space.
	output nextPutAll: self backdoorGetter;
		 nextPutAll: '() == '.
	variables isEmpty
		ifTrue: [output nextPutAll: 'skip']
		ifFalse: [output nextPutAll: 'return '.
			variables size > 1
				ifTrue: [output nextPutAll: 'mk_('].
			variables
				do: [:var | output nextPutAll: var]
				separatedBy: [output nextPut: $,].
			variables size > 1
				ifTrue: [output nextPutAll: ')']].
	output nextPutAll: ';';
		 cr.
	^ output contents! !

!SIVDM methodsFor: 'private' stamp: 'tomooda 1/11/2013 09:25'!
deleteTmpfile
	(tmpFilename notNil
			and: [FileStream isAFileNamed: tmpFilename])
		ifTrue: [FileDirectory
				deleteFilePath: (FileStream fullName: tmpFilename).
			tmpFilename := nil]! !

!SIVDM methodsFor: 'private' stamp: 'tomooda 12/4/2012 12:35'!
ensureStartupProcess
	process isNil
		ifTrue: [self startupProcess]! !

!SIVDM methodsFor: 'communicating' stamp: 'tomooda 11/2/2012 10:19'!
expression: aString ifError: errorReceivingBlock 
	| response |
	response := self trimPrompt: aString.
	^ (response isEmpty
			or: [response first = $=])
		ifTrue: [| expression |
			expression := response readStream next; next; upTo: Character cr.
			(expression notEmpty
					and: [expression last = Character cr])
				ifTrue: [expression := expression copyFrom: 1 to: expression size - 1].
			expression]
		ifFalse: [errorReceivingBlock value: response]! !

!SIVDM methodsFor: 'class access' stamp: 'TomohiroOda 7/24/2012 10:59'!
expressionParserClass 
	^ SIVDMValueParser! !

!SIVDM methodsFor: 'class access' stamp: 'TO 7/27/2012 17:32'!
expressionPrinterClass
	^ SIVDMExpressionPrinter! !

!SIVDM methodsFor: 'private' stamp: 'TO 10/19/2012 11:49'!
forkProcess: desc 
	(OSProcess accessor isExecutable: self javaPath)
			= true
		ifFalse: [| msg |
			msg := String new writeStream.
			msg nextPutAll: 'java not found at ';
				 nextPutAll: self javaPath;
				 nextPutAll: '.';
				 cr;
				 nextPutAll: 'Please specify its fullpath location in SIVDM class>>javaPath.'.
			SINotifier notify: msg contents.
			thisContext terminate].
	(OSProcess accessor
			isReadable: (FileStream fullName: self vdmjPath))
			= true
		ifFalse: [| msg |
			msg := String new writeStream.
			msg nextPutAll: 'The VDMJ interpreter not found at ';
				 nextPutAll: self vdmjPath;
				 nextPutAll: '.';
				 cr;
				 nextPutAll: 'Please specify its fullpath location in SIVDM class>>vdmjPath.';
				 cr;
				 nextPutAll: 'If you don''t have one, please visit http://sourceforge.net/projects/vdmj/ to download it.'.
			SINotifier notify: msg contents.
			thisContext terminate].
	^ UnixProcess
		forkJob: self javaPath
		arguments: {'-jar'. FileStream fullName: self vdmjPath. '-vdmsl'. '-i'. '-w'. '-q'}
		environment: nil
		descriptors: desc! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 11/8/2012 15:06'!
globalModuleName
	^ 'DEFAULT'! !

!SIVDM methodsFor: 'initialize-release' stamp: 'tomooda 1/2/2013 21:19'!
initialize
	super initialize.
	stateVariables := OrderedCollection new.
	originalSpecification := String new.
	generatedSpecification := String new.
	modulePositions := Dictionary new! !

!SIVDM methodsFor: 'accessing' stamp: 'TO 10/19/2012 11:31'!
javaPath
	^ self class javaPath! !

!SIVDM methodsFor: 'processing' stamp: 'TO 10/19/2012 09:58'!
load: aString 
	^ self
		load: aString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 11/2/2012 13:32'!
load: aString ifError: errorBlock
	^ self loadAll: (Array with: aString) ifError: errorBlock! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 11/2/2012 13:32'!
loadAll: anArrayOfString
	^ self
		loadAll: anArrayOfString
		ifError: [ :msg | 
			SINotifier notify: msg.
			nil ]! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 11/13/2012 10:00'!
loadAll: anArrayOfString ifError: errorBlock 
	| source |
	source := String new writeStream.
	anArrayOfString
		do: [:fileName | SIFileUtils
				readStream: fileName
				do: [:stream | 
					stream
						do: [:char | source nextPut: char].
					source nextPut: Character lf]].
	^ self specification: source contents ifError: errorBlock! !

!SIVDM methodsFor: 'accessing' stamp: 'tomooda 11/16/2012 10:55'!
moduleNames
	^ stateVariables
		collect: [:assoc | assoc key]! !

!SIVDM methodsFor: 'accessing' stamp: 'tomooda 1/2/2013 21:27'!
modulePositionAt: aString ifAbsent: errorBlock
	^ modulePositions at: aString ifAbsent: errorBlock! !

!SIVDM methodsFor: 'processing' stamp: 'TO 10/17/2012 14:46'!
plainEvaluate: aString limit: anInteger ifError: errorBlock 
	^ super
		plainEvaluate: 'print ' , aString
		limit: anInteger
		ifError: errorBlock! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'TO 11/26/2012 10:08'!
privateModuleStateAt: moduleNameString ifError: errorBlock 
	| stateString variableNames |
	variableNames := (stateVariables
				detect: [:assoc | assoc key = moduleNameString]
				ifNone: [^ errorBlock value: 'No such module: ' , moduleNameString]) value.
	self send: 'print ' , moduleNameString , '`' , self backdoorGetter , '()'.
	stateString := self
				expression: self readUpToPrompt
				ifError: [:msg | ^ errorBlock value: msg].
	^ variableNames isEmpty
		ifTrue: [Array new]
		ifFalse: [variableNames size = 1
				ifTrue: [Array with: variableNames first -> stateString]
				ifFalse: [| state |
					state := Array new writeStream.
					variableNames := variableNames readStream.
					self parser
						scanTuple: stateString readStream
						plainElementsDo: [:value | state nextPut: variableNames next -> value]
						ifError: [:msg | ^ errorBlock value: msg].
					state contents]]! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'tomooda 2/15/2013 08:39'!
privateModuleStateAt: moduleNameString put: anArray ifError: errorBlock 
	| variableNames stateString |
	variableNames := (stateVariables
				detect: [:assoc | assoc key = moduleNameString]
				ifNone: [^ errorBlock value: 'No such module: ' , moduleNameString]) value.
	stateString := variableNames isEmpty
				ifTrue: [String new]
				ifFalse: [variableNames size = 1
						ifTrue: [(anArray
								detect: [:assoc | assoc key = variableNames first]
								ifNone: [^ errorBlock value: 'Missing variable: ' , variableNames first]) value]
						ifFalse: [| tuple |
							tuple := String new writeStream.
							variableNames
								do: [:name | tuple nextPutAll: (anArray
											detect: [:assoc | assoc key = name]
											ifNone: [^ errorBlock value: 'Missing variable: ' , name]) value]
								separatedBy: [tuple nextPut: $,].
							tuple contents]].
	self send: 'default ' , moduleNameString.
	self readUpToPrompt.
	self send: 'print ' , self backdoorSetter , '(' , stateString , ')'.
	self
		expression: self readUpToPrompt
		ifError: [:msg | ^ errorBlock value: msg].
	self send: 'default ' , self globalModuleName.
	self readUpToPrompt.
	^ anArray! !

!SIVDM methodsFor: 'private' stamp: 'tomooda 1/11/2013 09:27'!
privateSpecification: aString ifError: errorBlock 
	originalSpecification := aString copy.
	SIFileUtils
		temporaryFile: 'SIVDM-specification:-'
		randomSuffixSize: 8
		suffix: '.vdm'
		do: [:name :stream | 
			stream ascii.
			stream lineEndConvention: #lf.
			stream nextPutAll: aString.
			stream close.
			self send: 'load ' , name.
			self
				expression: self readUpToPrompt
				ifError: [:msg | ^ errorBlock value: msg]].
	self deleteTmpfile.
	(SIFileUtils
		temporaryFileStream: 'SIVDM-specification:-'
		randomSuffixSize: 8
		suffix: '.vdm') in: [:stream |
			| name |
			name := stream name.
			tmpFilename := name.
			stream ascii.
			stream lineEndConvention: #lf.
			self
				createBackdoorsFrom: aString
				on: stream
				ifError: [:msg | ^ errorBlock value: msg].
			stream close.
			stream reset.
			generatedSpecification := stream contents.
			stream close.
			self send: 'load ' , name.
			self
				expression: self readUpToPrompt
				ifError: [:msg | ^ errorBlock value: msg].
			self send: 'default ' , self globalModuleName.
			self readUpToPrompt]! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/7/2012 14:27'!
read: aStream match: aString 
	aString
		do: [:char | 
			aStream atEnd
				ifTrue: [^ false].
			aStream peek = char
				ifFalse: [^ false].
			aStream next].
	^ true! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/7/2012 14:43'!
readNonSeparators: aStream 
	| term |
	term := String new writeStream.
	[aStream atEnd not
		and: [aStream peek isSeparator not]]
		whileTrue: [term nextPut: aStream next].
	^ term contents! !

!SIVDM methodsFor: 'communicating' stamp: 'tomooda 2/15/2013 08:38'!
readUpToPrompt
	| contents crStream |
	crStream := String new writeStream.
	[contents := self basicReadUpToPrompt.
	contents size > 1
		and: [(contents at: contents size - 1)
				= $]]]
		whileTrue: [self send: 'quit'].
	contents
		linesDo: [:line | 
			crStream nextPutAll: line.
			crStream cr].
	(reader notNil
			and: [reader peek = Character space])
		ifTrue: [reader next].
	^ crStream contents! !

!SIVDM methodsFor: 'communicating' stamp: 'tomooda 2/15/2013 08:48'!
readUpToPromptLimit: anInteger 
	^ anInteger
		ifNil: [self readUpToPrompt]
		ifNotNil: [| response |
			[response := super readUpToPromptLimit: anInteger.
			response size > 1
				and: [response last = $>
						and: [(response at: response size - 1)
								= $]]]]
				whileTrue: [self send: 'quit'].
			(reader notNil
					and: [reader peek = Character space])
				ifTrue: [reader next].
			response]! !

!SIVDM methodsFor: 'private' stamp: 'tomooda 12/7/2012 08:13'!
resumeSnapshot
	originalSpecification
		ifNotNil: [self
				privateSpecification: originalSpecification
				ifError: [:msg | ^ self]].
	snapshot
		ifNotNil: [snapshot
				do: [:moduleAndStateString | moduleAndStateString value
						ifNotNil: [self
								privateModuleStateAt: moduleAndStateString key
								put: moduleAndStateString value
								ifError: [:m | m]]]].
	snapshot := nil! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'TO 3/6/2013 13:32'!
scan: aStream modulesAndStatesAndPositionsDo: moduleAndStateAndExportPosEndPosBlock ifError: errorBlock 
	[aStream atEnd not]
		whileTrue: [| startPosition module exportsOrDefinitions hasExports exportPosition state |
			self
				skip: aStream
				upToKeyword: 'module'
				ifNone: [^ self].
			startPosition := aStream position - 'module' size + 1.
			self
				skipSeparators: aStream
				ifAtEnd: [^ errorBlock value: 'Expecting an identifier after "module".'].
			module := self readNonSeparators: aStream.
			exportsOrDefinitions := self
						skip: aStream
						upToAnyOfKeywords: #('exports' 'definitions' )
						ifNone: [^ errorBlock value: 'Expected "definitions" in a module.'].
			exportsOrDefinitions = 'exports'
				ifTrue: [| beginExportsPosition |
					hasExports := true.
					self
						skipSeparators: aStream
						ifAtEnd: [^ errorBlock value: 'Unexpected end of file.'].
					beginExportsPosition := aStream position.
					(self readNonSeparators: aStream)
							= 'all'
						ifTrue: [exportPosition := nil]
						ifFalse: [| importsOrDefinitions |
							importsOrDefinitions := self
										skip: aStream
										upToAnyOfKeywords: #('imports' 'definitions' )
										ifNone: [^ errorBlock value: 'Expected "definitions" in a module.'].
							aStream skip: (importsOrDefinitions size + 1) negated.
							self backSkipToNonSeparators: aStream.
							exportPosition := beginExportsPosition -> (aStream position + 1)]]
				ifFalse: [| insertPosition |
					insertPosition := aStream position - exportsOrDefinitions size.
					hasExports := false.
					exportPosition := insertPosition -> insertPosition].
			state := nil.
			[| stateOrEnd |
			stateOrEnd := self
						skip: aStream
						upToAnyOfKeywords: #('state' 'end' )
						ifNone: [^ self].
			stateOrEnd = 'state'
				ifTrue: [self
						skipSeparators: aStream
						ifAtEnd: [^ errorBlock value: 'Expecting an identifier after "state".'].
					state := self readNonSeparators: aStream].
			stateOrEnd = 'end'
				ifTrue: [| position forwardPosition |
					position := aStream position - 3.
					self
						skipSeparators: aStream
						ifAtEnd: [^ errorBlock value: 'Expecting an end of module.'].
					forwardPosition := aStream position.
					(self readNonSeparators: aStream)
							= module
						ifTrue: [state
								ifNil: [moduleAndStateAndExportPosEndPosBlock
										value: module
										value: Array new
										value: hasExports -> exportPosition
										value: position]
								ifNotNil: [| variables |
									variables := Array new writeStream.
									self send: 'default ' , module.
									self readUpToPrompt.
									self send: 'state'.
									self readUpToPrompt
										linesDo: [:line0 | 
											| pos |
											pos := line0 indexOf: Character space.
											pos > 1
												ifTrue: [ | line |
													line := line0 copyFrom: 1 to: pos
																	- 1.
													line = state
														ifFalse: [variables nextPut: line]]].
									variables := variables contents.
									moduleAndStateAndExportPosEndPosBlock
										value: module
										value: variables asSortedCollection asArray
										value: hasExports -> exportPosition
										value: position]]
						ifFalse: [aStream position: forwardPosition.
							stateOrEnd := nil]].
			stateOrEnd = 'end'] whileFalse.
		modulePositions at: module put: (startPosition to: aStream position)]! !

!SIVDM methodsFor: 'private-backdooring' stamp: 'TO 3/6/2013 13:48'!
scan: aStream variablesDo: variablesBlock ifError: errorBlock
	(self
		skip: aStream
		upToKeyword: 'state'
		ifNone: [])
		ifNil: [variablesBlock value: Array new]
		ifNotNil: [| state variables |
			self
				skipSeparators: aStream
				ifAtEnd: [^ errorBlock value: 'Expecting an identifier after "state".'].
			state := self readNonSeparators: aStream.
			variables := Array new writeStream.
			self send: 'state'.
			self readUpToPrompt
				linesDo: [:line0 | 
					| pos |
					pos := line0 indexOf: Character space.
					pos > 1
						ifTrue: [ | line |
							line := line0 copyFrom: 1 to: pos
											- 1.
							line = state
								ifFalse: [variables nextPut: line]]].
			variablesBlock value: variables contents asSortedCollection asArray].
		aStream upToEnd.
		modulePositions at: self globalModuleName put: (1 to: aStream position)
! !

!SIVDM methodsFor: 'private' stamp: 'TO 11/26/2012 10:17'!
shutDown
	self storeSnapshot.
	super shutDown! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/7/2012 15:34'!
skip: aStream upToAnyOfKeywords: anArrayOfString ifNone: errorBlock 
	[| position |
	self
		skipSeparators: aStream
		ifAtEnd: [^ errorBlock value].
	anArrayOfString
		do: [:keyword | 
			position := aStream position.
			((self read: aStream match: keyword)
					and: [aStream atEnd
							or: [aStream peek isSeparator]])
				ifTrue: [^ keyword].
			aStream position: position].
	self
		skipNonSeparators: aStream
		ifAtEnd: [^ errorBlock value]] repeat! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/7/2012 15:35'!
skip: aStream upToKeyword: aString ifNone: errorBlock 
	^ self
		skip: aStream
		upToAnyOfKeywords: (Array with: aString)
		ifNone: errorBlock! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/7/2012 14:39'!
skipNonSeparators: aStream ifAtEnd: errorBlock 
	[aStream atEnd not
		and: [aStream peek isSeparator not]]
		whileTrue: [aStream next].
	aStream atEnd
		ifTrue: [^ errorBlock value]! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/7/2012 14:37'!
skipSeparators: aStream ifAtEnd: errorBlock 
	[aStream atEnd not
		and: [aStream peek isSeparator]]
		whileTrue: [aStream next].
	aStream atEnd
		ifTrue: [^ errorBlock value]! !

!SIVDM methodsFor: 'accessing' stamp: 'tomooda 1/2/2013 13:58'!
specification
	^ originalSpecification! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 1/10/2013 14:18'!
specification: aString ifError: errorBlock 
	self
		critical: [self ensureStartupProcess.
			self privateSpecification: aString ifError: errorBlock]! !

!SIVDM methodsFor: 'private' stamp: 'TO 11/26/2012 10:28'!
startupProcess
	super startupProcess.
	reader peek = Character space
		ifTrue: [reader next].
	self resumeSnapshot! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 1/10/2013 14:18'!
stateAt: moduleNameString 
	^ self
		critical: [self ensureStartupProcess.
			self
				privateModuleStateAt: moduleNameString
				ifError: [:msg | 
					SINotifier notify: msg.
					nil]]! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 1/10/2013 14:18'!
stateAt: moduleNameString put: anArray 
	^ self
		critical: [self ensureStartupProcess.
			self
				privateModuleStateAt: moduleNameString
				put: anArray
				ifError: [:msg | 
					SINotifier notify: msg.
					nil]]! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 1/10/2013 14:18'!
states
	^ self
		critical: [self ensureStartupProcess.
			Dictionary
				withAll: (stateVariables
						collect: [:moduleAndVariables | moduleAndVariables key
								-> (self
										privateModuleStateAt: moduleAndVariables key
										ifError: [:msg | msg])])]! !

!SIVDM methodsFor: 'private' stamp: 'TO 11/26/2012 10:18'!
storeSnapshot
	snapshot := process
				ifNotNil: [stateVariables
						collect: [:moduleAndVariables | moduleAndVariables key
								-> (self
										privateModuleStateAt: moduleAndVariables key
										ifError: [:msg | msg])]]! !

!SIVDM methodsFor: 'processing' stamp: 'tomooda 1/10/2013 14:18'!
swapSpecification: aString ifError: errorBlock 
	self
		critical: [| oldSpec oldStates oldStateVariables newStates |
			self ensureStartupProcess.
			oldSpec := originalSpecification copy.
			oldStateVariables := stateVariables.
			oldStates := stateVariables
						collect: [:moduleAndVariables | moduleAndVariables key
								-> (self
										privateModuleStateAt: moduleAndVariables key
										ifError: [:msg | 
											self close.
											^ errorBlock value: msg])].
			self
				privateSpecification: aString
				ifError: [:msg | 
					self startupProcess.
					self
						privateSpecification: oldSpec
						ifError: [:m | 
							self startupProcess.
							^ errorBlock value: m , String cr , '---' , String cr , msg].
					oldStates
						do: [:moduleAndStateString | self
								privateModuleStateAt: moduleAndStateString key
								put: moduleAndStateString value
								ifError: [:m | ^ errorBlock value: m , String cr , '---' , String cr , msg]].
					^ errorBlock value: msg].
			newStates := oldStates
						select: [:moduleAndState | stateVariables
								contains: [:assoc | assoc key = moduleAndState key]].
			newStates := newStates
						collect: [:moduleAndState | 
							| oldState newVariables |
							oldState := (oldStates
										detect: [:assoc | assoc key = moduleAndState key]
										ifNone: []) value.
							newVariables := (stateVariables
										detect: [:assoc | assoc key = moduleAndState key]
										ifNone: []) value.
							moduleAndState key
								-> (newVariables
										collect: [:variable | variable -> (oldState
													detect: [:assoc | assoc key = variable]
													ifNone: [variable -> variable]) value])].
			newStates
				do: [:moduleAndStateString | self
						privateModuleStateAt: moduleAndStateString key
						put: moduleAndStateString value
						ifError: [:msg | msg]]]! !

!SIVDM methodsFor: 'private' stamp: 'tomooda 1/11/2013 08:26'!
tmpFilename
	^ tmpFilename! !

!SIVDM methodsFor: 'accessing' stamp: 'TO 10/19/2012 11:31'!
vdmjPath
	^ self class vdmjPath! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/13/2012 10:18'!
whiten: aString 
	| output |
	output := String new writeStream.
	self whiten: aString readStream on: output.
	^ output contents! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/9/2012 08:47'!
whiten: aReadStream on: aWriteStream 
	| whitener |
	whitener := #whitenInNormal:on:.
	[aReadStream atEnd]
		whileFalse: [whitener := self
						perform: whitener
						with: aReadStream
						with: aWriteStream]! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 12/4/2012 09:21'!
whitenInMultilineComment: aReadStream on: aWriteStream 
	| char |
	char := aReadStream next.
	(char = $*
			and: [aReadStream atEnd not
					and: [aReadStream peek = $/]])
		ifTrue: [aReadStream next.
			aWriteStream nextPutAll: '  '.
			^ #whitenInNormal:on:].
	aWriteStream nextPut: Character space.
	^ #whitenInMultilineComment:on:! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 12/4/2012 09:22'!
whitenInNormal: aReadStream on: aWriteStream 
	| char |
	char := aReadStream next.
	char = $"
		ifTrue: [aWriteStream nextPut: $".
			^ #whitenInString:on:].
	(char = $-
			and: [aReadStream atEnd not
					and: [aReadStream peek = $-]])
		ifTrue: [aReadStream next.
			aWriteStream nextPutAll: '  '.
			^ #whitenInSinglelineComment:on:].
	(char = $/
			and: [aReadStream atEnd not
					and: [aReadStream peek = $*]])
		ifTrue: [aReadStream next.
			aWriteStream nextPutAll: '  '.
			^ #whitenInMultilineComment:on:].
	aWriteStream nextPut: char.
	^ #whitenInNormal:on:! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/9/2012 09:15'!
whitenInSinglelineComment: aReadStream on: aWriteStream 
	| char |
	char := aReadStream next.
	char = Character lf
		ifTrue: [aWriteStream nextPut: Character lf.
			^ #whitenInNormal:on:].
	char = Character cr
		ifTrue: [aWriteStream nextPut: Character cr.
			^ #whitenInNormal:on:].
	aWriteStream nextPut: Character space.
	^ #whitenInSinglelineComment:on:! !

!SIVDM methodsFor: 'private-streaming' stamp: 'tomooda 11/9/2012 08:26'!
whitenInString: aReadStream on: aWriteStream 
	| char |
	char := aReadStream next.
	char = $"
		ifTrue: [aWriteStream nextPut: $".
			^ #whitenInNormal:on:].
	(char = $\
			and: [aReadStream atEnd not
					and: [aReadStream peek = $"]])
		ifTrue: [aReadStream next.
			aWriteStream nextPutAll: '  '.
			^ #whitenInString:on:].
	aWriteStream nextPut: Character space.
	^ #whitenInString:on:! !

TestCase subclass: #SIVDMParserPrinterTest
	instanceVariableNames: 'parser printer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Test'!

!SIVDMParserPrinterTest methodsFor: 'running' stamp: 'TO 7/25/2012 15:07'!
setUp
	parser := SIVDMValueParser new.
	printer := SIVDMExpressionPrinter new! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:07'!
test: anObject 
	self
		assert: anObject
		equals: (parser
				parse: (printer print: anObject))! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:27'!
testBoolean
	self test: true.
	self test: false! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:07'!
testCharacter
	self test: $a.
	self test: $z! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 14:12'!
testClassMap
	printer
		addClassMap: #Point
		to: [:point :vdmStream | 
			vdmStream nextPutAll: 'mk_Point('.
			point x printOn: vdmStream.
			vdmStream nextPutAll: ','.
			point y printOn: vdmStream.
			vdmStream nextPut: $)].
	parser
		addAlgebraicDataMap: 'mk_Point'
		to: [:constructor :args | args first @ args second].
	self test: 1 @ 2! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:07'!
testInteger
	self
		assert: 441
		equals: (parser parseInteger: '441' readStream).
	self
		assert: -441
		equals: (parser parseInteger: '-441' readStream)! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:07'!
testList
	self test: #(4 4 1 ) asOrderedCollection.
	self test: #(4 ) asOrderedCollection.
	self test: #() asOrderedCollection! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:07'!
testLowerIdentifierChar
	self
		assert: (parser isLowerIdentifierChar: $a).
	self
		assert: (parser isLowerIdentifierChar: $1).
	self
		assert: (parser isLowerIdentifierChar: $_).
	self
		deny: (parser isLowerIdentifierChar: $A).
	self
		deny: (parser isLowerIdentifierChar: $+)! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:08'!
testNat
	self
		assert: 441
		equals: (parser parseNaturalNumber: '441' readStream)! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:08'!
testNumber
	self test: 1.
	self test: -2.
	self test: 3.4.
	self test: -5.6.
	self test: -7.8e-9! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:31'!
testParen
	self assert: (parser parse: '1') equals: (parser parse: '((1))').
	self assert: (parser parse: 'mk_(1)') equals: (parser parse: '(mk_((1)))')! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 14:02'!
testParseErrorMessage
	| message |
	message := 'abc
def
'.
	self
		assert: message
		equals: (parser parseErrorMessage: (message , '> ') readStream)! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:29'!
testQuote
	self test: #Quote.
	self test: #'Like this'! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:29'!
testRecord
	self
		test: (SIVDMRecordValue of: 'mk_Freq' withAll: #(4 4 1 )).
	self
		test: (SIVDMRecordValue of: 'mk_Freq')! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:24'!
testSet
	self test: #(4 4 1 ) asSet.
	self test: #(4 ) asSet.
	self test: #() asSet! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:08'!
testString
	self test: 'Hello world'.
	self test: 'How''ya doing?\nI''m fine, thanks!!'! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:29'!
testToken
	self
		test: (SIVDMTokenValue with: 1).
	self test: (SIVDMTokenValue with: #(4 4 1))! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 15:08'!
testTuple
	self test: #(4 4 1 ).
	self test: #('Freq' 25 )! !

!SIVDMParserPrinterTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 13:55'!
testUnit
	self test: nil! !

TestCase subclass: #SIVDMRecordValueTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Test'!

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:47'!
testArgs
	self assert: #() equals: (SIVDMRecordValue of: 'fure') args.
	self assert: #(4 4 1 ) equals: (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 )) args! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:47'!
testConstructor
	self assert: 'fure' equals: (SIVDMRecordValue of: 'fure') constructor.
	self assert: 'freq' equals: (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 )) constructor! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:48'!
testEquals
	self assert: (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 ))
			= (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 )).
	self deny: (SIVDMRecordValue of: 'Freq' withAll: #(4 4 1 ))
			= (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 )).
	self deny: (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 ))
			= (SIVDMRecordValue of: 'freq' withAll: #(441 ))! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 13:56'!
testFirst
	self assert: 4 equals: (SIVDMRecordValue of: 'freq' withAll: #(4 1 2 6)) first! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:48'!
testHash
	self assert: (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 )) hash equals: (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 )) hash! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 13:57'!
testLast
	self assert: 6 equals: (SIVDMRecordValue of: 'freq' withAll: #(4 1 2 6)) last! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 20:13'!
testPrintString
	self assert: 'fure()' equals: (SIVDMRecordValue of: 'fure') printString.
	self assert: 'freq(4, 4, 1)' equals: (SIVDMRecordValue of: 'freq' withAll: #(4 4 1 )) printString! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 13:56'!
testSecond
	self assert: 1 equals: (SIVDMRecordValue of: 'freq' withAll: #(4 1 2 6)) second! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:48'!
testStoreString
	| data |
	data := SIVDMRecordValue of: 'freq' withAll: #(4 4 1 ).
	self
		assert: data
		equals: (Compiler evaluate: data storeString)! !

!SIVDMRecordValueTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 13:56'!
testThird
	self assert: 2 equals: (SIVDMRecordValue of: 'freq' withAll: #(4 1 2 6)) third! !

TestCase subclass: #SIVDMTest
	instanceVariableNames: 'vdm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Test'!

!SIVDMTest methodsFor: 'running' stamp: 'TO 7/26/2012 11:28'!
setUp
	vdm := SIVDM new! !

!SIVDMTest methodsFor: 'running' stamp: 'TO 7/26/2012 11:28'!
tearDown
	vdm close.
	vdm := nil! !

!SIVDMTest methodsFor: 'testing' stamp: 'TO 7/26/2012 11:29'!
testArithmetic
	self
		assert: 7
		equals: (vdm evaluate: '1+2*3')! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 12:10'!
testAsVDMConstant
	| hello |
	vdm close.
	vdm := SIVDM specification: 'values
	hello : seq of char = "hello"'.
	hello := 'hello' asVDMConstantIn: vdm.
	self assert: 'hello' equals: hello value.
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 12:06'!
testAsVDMExpression
	| onePlusTwo |
	vdm close.
	vdm := SIVDM plain.
	onePlusTwo := '1 + 2' asVDMExpressionIn: vdm.
	self assert: 3 equals: onePlusTwo value.
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 12:20'!
testAsVDMFunction1
	| f |
	vdm close.
	vdm := SIVDM specification: '
	/* returns the double of the argument */
	functions
		f : int -> int
		f(x) == x * 2'.
	f := 'f' asVDMFunctionIn: vdm.
	self
		assert: 2
		equals: (f value: 1).
	self
		assert: 4
		equals: (f value: 2).
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 12:20'!
testAsVDMFunction2
	| f |
	vdm close.
	vdm := SIVDM specification: '
	/* returns the double of the argument */
	functions
		f : int*int -> int
		f(x,y) == x * y'.
	f := 'f' asVDMFunctionIn: vdm.
	self
		assert: 2
		equals: (f value: 1 value: 2).
	self
		assert: 6
		equals: (f value: 2 value: 3).
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 12:20'!
testAsVDMFunction3
	| f |
	vdm close.
	vdm := SIVDM specification: '
	/* returns the double of the argument */
	functions
		f : int*int*int -> int
		f(x,y,z) == x * y + z'.
	f := 'f' asVDMFunctionIn: vdm.
	self
		assert: 5
		equals: (f
				value: 1
				value: 2
				value: 3).
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 12:20'!
testAsVDMFunction4
	| f |
	vdm close.
	vdm := SIVDM specification: '
	/* returns the double of the argument */
	functions
		f : int*int*int -> int
		f(x,y,z) == x * y + z'.
	f := 'f' asVDMFunctionIn: vdm.
	self
		assert: 5
		equals: (f valueWithArguments: #(1 2 3 )).
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:04'!
testAsVDMOperation
	| countUp |
	vdm close.
	vdm := SIVDM specification: 'module Counter
definitions
state counter of
	count : int
init
	s == s = mk_counter(1)
end
operations
	countUp : () ==> int -- aka increment
	countUp() == (count := count + 1; return count)
end Counter'.
	countUp := 'Counter`countUp' asVDMOperationIn: vdm.
	self assert: 2 equals: countUp value.
	self assert: 3 equals: countUp value.
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 11:55'!
testAsVDMVariable
	| countUp count |
	vdm close.
	vdm := SIVDM specification: '
state counter of
	count : int
init
	s == s = mk_counter(1)
end
operations
	countUp : () ==> int
	countUp() == (count := count + 1; return count)'.
	countUp := 'countUp' asVDMOperationIn: vdm.
	count := 'count' asVDMVariableIn: vdm.
	self assert: 1 equals: count value.
	countUp value.
	self assert: 2 equals: count value.
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/12/2012 13:56'!
testLoad
	SIFileUtils
		temporaryFile: 'SIVDMTest-testLoad-'
		randomSuffixSize: 8
		suffix: '.vdm'
		do: [:filename :stream | 
			stream nextPutAll: 'functions';
				 lf.
			stream nextPutAll: '  fizzbuzz : int -> token';
				 lf.
			stream nextPutAll: '  fizzbuzz(x) == let fb = fizz(x)^buzz(x) in if fb = "" then mk_token(x) else mk_token(fb);';
				 lf.
			stream lf.
			stream nextPutAll: '  fizz : int -> seq of char';
				 lf.
			stream nextPutAll: '  fizz(x) == if x mod 3 = 0 then "fizz" else "";';
				 lf.
			stream lf.
			stream nextPutAll: '  buzz : int -> seq of char';
				 lf.
			stream nextPutAll: '  buzz(x) == if x mod 5 = 0 then "buzz" else "";';
				 lf.
			stream close.
			vdm close.
			vdm := SIVDM load: filename.
			self assert: 13 equals: (vdm evaluate: 'fizzbuzz(13)') value.
			self assert: 'fizz' equals: (vdm evaluate: 'fizzbuzz(6)') value.
			self assert: 'buzz' equals: (vdm evaluate: 'fizzbuzz(20)') value.
			self assert: 'fizzbuzz' equals: (vdm evaluate: 'fizzbuzz(30)') value.
			vdm close]! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 13:59'!
testLoadAll
	SIFileUtils
		temporaryFile: 'SIVDMTest-testLoad-'
		randomSuffixSize: 8
		suffix: '.vdm'
		do: [:filename :stream | 
			stream nextPutAll: 'functions';
				 lf.
			stream nextPutAll: '  fizz : int -> seq of char';
				 lf.
			stream nextPutAll: '  fizz(x) == if x mod 3 = 0 then "fizz" else "";';
				 lf.
			stream lf.
			stream nextPutAll: '  buzz : int -> seq of char';
				 lf.
			stream nextPutAll: '  buzz(x) == if x mod 5 = 0 then "buzz" else "";';
				 lf.
			stream close.
			SIFileUtils
				temporaryFile: 'SIVDMTest-testLoad-'
				randomSuffixSize: 8
				suffix: '.vdm'
				do: [:filename2 :stream2 | 
					stream2 nextPutAll: 'functions';
						 lf.
					stream2 nextPutAll: '  fizzbuzz : int -> token';
						 lf.
					stream2 nextPutAll: '  fizzbuzz(x) == let fb = fizz(x)^buzz(x) in if fb = "" then mk_token(x) else mk_token(fb);';
						 lf.
					stream2 close.
					vdm
						loadAll: (Array with: filename with: filename2).
					self assert: 13 equals: (vdm evaluate: 'fizzbuzz(13)') value.
					self assert: 'fizz' equals: (vdm evaluate: 'fizzbuzz(6)') value.
					self assert: 'buzz' equals: (vdm evaluate: 'fizzbuzz(20)') value.
					self assert: 'fizzbuzz' equals: (vdm evaluate: 'fizzbuzz(30)') value.
					vdm close]]! !

!SIVDMTest methodsFor: 'testing' stamp: 'TO 7/26/2012 11:29'!
testMultiArithmetic
	self
		assert: 7
		equals: (vdm evaluate: '1+2*3').
	self
		assert: 34
		equals: (vdm evaluate: '4+5*6')! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 14:25'!
testParse
	self
		assert: (OrderedCollection with: 1)
		equals: (vdm parse: '[1]')! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 14:23'!
testPrint
	self
		assert: '[1]'
		equals: (vdm print: (OrderedCollection with: 1))! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/13/2012 11:19'!
testSpecification1
	vdm close.
	vdm := SIVDM specification: '
functions
  fizzbuzz : int -> token
  fizzbuzz(x) == let fb = fizz(x)^buzz(x) in if fb = "" then mk_token(x) else mk_token(fb);
  fizz : int -> seq of char
  fizz(x) == if x mod 3 = 0 then "fizz" else "";
  buzz : int -> seq of char
  buzz(x) == if x mod 5 = 0 then "buzz" else "";'.
	self assert: 13 equals: (vdm evaluate: 'fizzbuzz(13)') value.
	self assert: 'fizz' equals: (vdm evaluate: 'fizzbuzz(6)') value.
	self assert: 'buzz' equals: (vdm evaluate: 'fizzbuzz(20)') value.
	self assert: 'fizzbuzz' equals: (vdm evaluate: 'fizzbuzz(30)') value.
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:05'!
testSpecification2
	vdm close.
	vdm := SIVDM specification: 'module FizzBuzzGame
definitions
state counter of
	count : int
init
	s == s = mk_counter(1)
end
functions
  fizzbuzz : int -> token
  fizzbuzz(x) == let fb = fizz(x)^buzz(x) in if fb = "" then mk_token(x) else mk_token(fb);
  fizz : int -> seq of char
  fizz(x) == if x mod 3 = 0 then "fizz" else "";
  buzz : int -> seq of char
  buzz(x) == if x mod 5 = 0 then "buzz" else "";
end FizzBuzzGame
'.
	self assert: 13 equals: (vdm evaluate: 'FizzBuzzGame`fizzbuzz(13)') value.
	self assert: 'fizz' equals: (vdm evaluate: 'FizzBuzzGame`fizzbuzz(6)') value.
	self assert: 'buzz' equals: (vdm evaluate: 'FizzBuzzGame`fizzbuzz(20)') value.
	self assert: 'fizzbuzz' equals: (vdm evaluate: 'FizzBuzzGame`fizzbuzz(30)') value.
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:06'!
testSpecification3
	vdm close.
	vdm := SIVDM specification: 'module FizzBuzzGame
exports operations next:()==>int;
definitions
state counter of
	count : int
init
	s == s = mk_counter(1)
end
operations
	increment : () ==> ()
	increment() == count := count + 1;
	next : () ==> int
	next() == (increment();return count);
end FizzBuzzGame
'.
	self assert: 2 equals: (vdm evaluate: 'FizzBuzzGame`next()').
	self assert: 3 equals: (vdm evaluate: 'FizzBuzzGame`next()').
	vdm close! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:10'!
testStateAtPutFlat1
	vdm specification: '
	functions
		f : int -> int
		f(x) == x * 2;
		' ifError: [ :msg | self fail].
	self assert: #() equals: (vdm stateAt: 'DEFAULT').
	vdm stateAt: 'DEFAULT' put: Array new.
	self assert: #() equals: (vdm stateAt: 'DEFAULT').! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:10'!
testStateAtPutFlat2
	vdm
		specification: '
	state foo of
		bar : int
	init
		b == b = mk_foo(1)
	end
	functions
		f : int -> int
		f(x) == x * 2;
		'
		ifError: [:msg | self fail].
	self
		assert: {'bar' -> '1'}
		equals: (vdm stateAt: 'DEFAULT').
	vdm stateAt: 'DEFAULT' put: {'bar' -> '2'}.
	self
		assert: {'bar' -> '2'}
		equals: (vdm stateAt: 'DEFAULT')! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:38'!
testStateAtPutFlat3
	vdm
		specification: '
	state foo of
		bar : int
		buzz : seq of char
	inv
		mk_foo(bar, buzz) == bar = len buzz
	init
		b == b = mk_foo(1, "2")
	end
	functions
		f : int -> int
		f(x) == x * 2;
		'
		ifError: [:msg | self fail].
	self assert: {'bar' -> '1'. 'buzz' -> '"2"'} asSortedCollection equals: (vdm stateAt: 'DEFAULT') asSortedCollection.
	vdm stateAt: 'DEFAULT' put: {'bar' -> '3'. 'buzz' -> '"456"'}.
	self assert: {'bar' -> '3'. 'buzz' -> '"456"'} asSortedCollection equals: (vdm stateAt: 'DEFAULT') asSortedCollection! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:11'!
testStateAtPutModule1
	vdm specification: '
module A
definitions
	functions
		f : int -> int
		f(x) == x * 2;
end A		' ifError: [ :msg | self fail].
	self assert: #() equals: (vdm stateAt: 'A').
	vdm stateAt: 'A' put: Array new.
	self assert: #() equals: (vdm stateAt: 'A').! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:12'!
testStateAtPutModule2
	vdm
		specification: '
module A
definitions
	state foo of
		bar : int
	init
		b == b = mk_foo(1)
	end
	functions
		f : int -> int
		f(x) == x * 2;
end A'
		ifError: [:msg | self fail].
	self
		assert: {'bar' -> '1'}
		equals: (vdm stateAt: 'A').
	vdm stateAt: 'A' put: {'bar' -> '2'}.
	self
		assert: {'bar' -> '2'}
		equals: (vdm stateAt: 'A')! !

!SIVDMTest methodsFor: 'testing' stamp: 'tomooda 11/14/2012 15:37'!
testStateAtPutModule3
	vdm
		specification: '
module A
definitions
	state foo of
		bar : int
		buzz : seq of char
	inv
		mk_foo(bar, buzz) == bar = len buzz
	init
		b == b = mk_foo(1, "2")
	end
	functions
		f : int -> int
		f(x) == x * 2;
end A'
		ifError: [:msg | self fail].
	self assert: {'bar' -> '1'. 'buzz' -> '"2"'} asSortedCollection equals: (vdm stateAt: 'A') asSortedCollection.
	vdm stateAt: 'A' put: {'bar' -> '3'. 'buzz' -> '"456"'}.
	self assert: {'bar' -> '3'. 'buzz' -> '"456"'} asSortedCollection equals: (vdm stateAt: 'A') asSortedCollection! !

TestCase subclass: #SIVDMTokenValueTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Test'!

!SIVDMTokenValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:50'!
testEquals
	self assert: (SIVDMTokenValue with: 'freq')
			= (SIVDMTokenValue with: 'freq').
	self deny: (SIVDMTokenValue with: 'freq')
			= (SIVDMTokenValue with: 'frq').
	self deny: (SIVDMTokenValue with: 'freq')
			= (SIVDMTokenValue with: true)! !

!SIVDMTokenValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:51'!
testHash
	self assert: (SIVDMTokenValue with: 'freq') hash equals: (SIVDMTokenValue with: 'freq') hash! !

!SIVDMTokenValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:53'!
testPrintString
	self assert: 'mk_token(''fure'')' equals: (SIVDMTokenValue with: 'fure') printString! !

!SIVDMTokenValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 20:15'!
testStoreString
	| data |
	data := SIVDMTokenValue with: 'fure'.
	self assert: data equals: (Compiler evaluate: (data) storeString)! !

!SIVDMTokenValueTest methodsFor: 'testing' stamp: 'TO 7/25/2012 17:50'!
testValue
	self assert: 'fure' equals: (SIVDMTokenValue with: 'fure') value! !

Object subclass: #FizzBuzz
	instanceVariableNames: 'vdm count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Process'!

!FizzBuzz methodsFor: 'operating' stamp: 'tomooda 12/11/2012 16:03'!
count
	^ count! !

!FizzBuzz methodsFor: 'operating' stamp: 'tomooda 12/11/2012 16:03'!
count: anInteger
	count := anInteger! !

!FizzBuzz methodsFor: 'operating' stamp: 'tomooda 12/11/2012 16:04'!
fizz
	^ count % 3 = 0! !

!FizzBuzz methodsFor: 'initialize-release' stamp: 'tomooda 12/11/2012 16:00'!
initialize
	super initialize.
	vdm := SIVDM spec: '
/*
The FizzBuzz game counts numbers up,
saying Fizz when the number is divisible by 3,
Buzz when the number is divisible by 5,
FizzBuzz when the number is divisible by 15,
otherwise the number.
*/
state counter of
  count : int
init
  -- the game starts with "1".
  s == s = mk_counter(1)
end

functions
  -- say fizz when divisible by 3
  fizz : int -> seq of char
  fizz(x) == if x mod 3 = 0 then "fizz" else "";

  -- say buzz when divisible by 5
  buzz : int -> seq of char
  buzz(x) == if x mod 5 = 0 then "buzz" else "";

  -- say fizzbuzz when divisible by 15, otherwise the number.
  fizzbuzz : int -> token
  fizzbuzz(x) == let fb = fizz(x) ^ buzz(x) in if fb = "" then mk_token(x) else mk_token(fb);

operations
  -- A player says "fizz". Answers whether or not it is legitimate call.
  sayFizz : () ==> bool
  sayFizz() == if fizzbuzz(count) = mk_token("fizz") then (count := count + 1; return true) else return false; 
  -- A player says "buzz". Answers whether or not it is legitimate call.
  sayBuzz : () ==> bool
  sayBuzz() == if fizzbuzz(count) = mk_token("buzz") then (count := count + 1; return true) else return false; 
  -- A player says "fizzbuzz". Answers whether or not it is legitimate call.
  sayFizzBuzz : () ==> bool
  sayFizzBuzz() == if fizzbuzz(count) = mk_token("fizzbuzz") then (count := count + 1; return true) else return false; 
  -- A player says a number. Answers whether or not it is legitimate call.
  sayNumber : () ==> bool
  sayNumber() == if fizzbuzz(count) = mk_token(count) then (count := count + 1; return true) else return false; 

'.! !

!FizzBuzz methodsFor: 'operating' stamp: 'tomooda 12/11/2012 16:04'!
sayFizz
	^ self fizz == 'fizz' ifTrue: [^true]! !

Object subclass: #SIVDMDefinition
	instanceVariableNames: 'types values functions state operations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Specification'!

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:50'!
functionAt: aString ifAbsent: errorBlock
	^ functions at: aString ifAbsent: errorBlock! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:10'!
functionAt: nameString put: soourceString 
	^ self
		functionAt: nameString
		put: soourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 10:44'!
functionAt: nameString put: soourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := functions
				at: nameString
				ifAbsent: [].
	functions at: nameString put: soourceString.
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNil: [functions removeKey: nameString]
				ifNotNil: [functions at: nameString put: oldSpec].
			^ errorBlock value: msg].
	^ soourceString! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/24/2012 10:32'!
functionNames
	^ functions keys asSortedCollection asArray! !

!SIVDMDefinition methodsFor: 'initialize-release' stamp: 'tomooda 12/3/2012 11:11'!
initialize
	super initialize.
	types := Dictionary new.
	values := Dictionary new.
	functions := Dictionary new.
	state := nil.
	operations := Dictionary new! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:51'!
operationAt: aString ifAbsent: errorBlock
	^ operations at: aString ifAbsent: errorBlock! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:10'!
operationAt: nameString put: soourceString 
	^ self
		operationAt: nameString
		put: soourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 10:45'!
operationAt: nameString put: soourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := operations
				at: nameString
				ifAbsent: [].
	operations at: nameString put: soourceString.
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNil: [operations removeKey: nameString]
				ifNotNil: [operations at: nameString put: oldSpec].
			^ errorBlock value: msg].
	^ soourceString! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/24/2012 10:32'!
operationNames
	^ operations keys asSortedCollection asArray! !

!SIVDMDefinition methodsFor: 'printing' stamp: 'tomooda 12/3/2012 11:24'!
printSpecificationOn: aStream 
	types notEmpty
		ifTrue: [aStream nextPutAll: 'types';
				 nextPut: Character cr.
			types keys asSortedCollection
				do: [:typeName | aStream
						nextPutAll: (types at: typeName);
						 nextPut: Character cr].
			aStream nextPut: Character cr].
	values notEmpty
		ifTrue: [aStream nextPutAll: 'values';
				 nextPut: Character cr.
			values keys asSortedCollection
				do: [:typeName | 
					aStream
						nextPutAll: (values at: typeName).
					aStream nextPut: Character cr].
			aStream nextPut: Character cr].
	functions notEmpty
		ifTrue: [aStream nextPutAll: 'functions';
				 nextPut: Character cr.
			functions keys asSortedCollection
				do: [:typeName | aStream
						nextPutAll: (functions at: typeName);
						 nextPut: Character cr].
			aStream nextPut: Character cr].
	state
		ifNotNil: [aStream nextPutAll: state value.
			aStream nextPut: Character cr].
	operations notEmpty
		ifTrue: [aStream nextPutAll: 'operations'.
			aStream nextPut: Character cr.
			operations keys asSortedCollection
				do: [:typeName | 
					aStream
						nextPutAll: (operations at: typeName).
					aStream nextPut: Character cr].
			aStream nextPut: Character cr]! !

!SIVDMDefinition methodsFor: 'processing' stamp: 'tomooda 11/21/2012 11:53'!
recompileIfError: errorBlock
	^ self subclassResponsibility! !

!SIVDMDefinition methodsFor: 'removing' stamp: 'tomooda 11/21/2012 11:47'!
removeFunction: aString ifError: errorBlock 
	| oldSpec result |
	oldSpec := functions
				at: aString
				ifAbsent: [].
	result := functions
				removeKey: aString
				ifAbsent: [].
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNotNil: [functions at: aString put: oldSpec].
			^ errorBlock value: msg].
	^ result! !

!SIVDMDefinition methodsFor: 'removing' stamp: 'tomooda 11/21/2012 11:48'!
removeOperation: aString ifError: errorBlock 
	| oldSpec result |
	oldSpec := operations
				at: aString
				ifAbsent: [].
	result := operations
				removeKey: aString
				ifAbsent: [].
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNotNil: [operations at: aString put: oldSpec].
			^ errorBlock value: msg].
	^ result! !

!SIVDMDefinition methodsFor: 'removing' stamp: 'tomooda 11/21/2012 11:49'!
removeStateIfError: errorBlock 
	| oldSpec |
	oldSpec := state.
	state := nil.
	self
		recompileIfError: [:msg | 
			state := oldSpec.
			^ errorBlock value: msg].
	^ oldSpec
		ifNotNil: [oldSpec key]! !

!SIVDMDefinition methodsFor: 'removing' stamp: 'tomooda 11/21/2012 11:47'!
removeType: aString ifError: errorBlock 
	| oldSpec result |
	oldSpec := types
				at: aString
				ifAbsent: [].
	result := types
				removeKey: aString
				ifAbsent: [].
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNotNil: [types at: aString put: oldSpec].
			^ errorBlock value: msg].
	^ result! !

!SIVDMDefinition methodsFor: 'removing' stamp: 'tomooda 11/21/2012 11:47'!
removeValue: aString ifError: errorBlock 
	| oldSpec result |
	oldSpec := values
				at: aString
				ifAbsent: [].
	result := values
				removeKey: aString
				ifAbsent: [].
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNotNil: [values at: aString put: oldSpec].
			^ errorBlock value: msg].
	^ result! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:52'!
stateAt: aString ifAbsent: errorBlock 
	^ (state notNil
		and: [state key = aString])
		ifTrue: [state value]
		ifFalse: errorBlock! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:10'!
stateAt: nameString put: soourceString 
	^ self
		stateAt: nameString
		put: soourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 11:18'!
stateAt: nameString put: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := state.
	state := nameString->sourceString.
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNil: [state := nil]
				ifNotNil: [state := oldSpec].
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 09:45'!
stateName
	^ state ifNil: [nil] ifNotNil: [state key]! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:50'!
typeAt: aString ifAbsent: errorBlock
	^ types at: aString ifAbsent: errorBlock! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:10'!
typeAt: nameString put: soourceString 
	^ self
		typeAt: nameString
		put: soourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 10:45'!
typeAt: nameString put: soourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := types
				at: nameString
				ifAbsent: [].
	types at: nameString put: soourceString.
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNil: [types removeKey: nameString]
				ifNotNil: [types at: nameString put: oldSpec].
			^ errorBlock value: msg].
	^ soourceString! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/24/2012 10:32'!
typeNames
	^ types keys asSortedCollection asArray! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:51'!
valueAt: aString ifAbsent: errorBlock
	^ values at: aString ifAbsent: errorBlock! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:10'!
valueAt: nameString put: soourceString 
	^ self
		valueAt: nameString
		put: soourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 10:45'!
valueAt: nameString put: soourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := values
				at: nameString
				ifAbsent: [].
	values at: nameString put: soourceString.
	self
		recompileIfError: [:msg | 
			oldSpec
				ifNil: [values removeKey: nameString]
				ifNotNil: [values at: nameString put: oldSpec].
			^ errorBlock value: msg].
	^ soourceString! !

!SIVDMDefinition methodsFor: 'accessing' stamp: 'tomooda 11/24/2012 10:32'!
valueNames
	^ values keys asSortedCollection asArray! !

SIVDMDefinition subclass: #SIVDMFlatSpecification
	instanceVariableNames: 'models'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Specification'!

!SIVDMFlatSpecification methodsFor: 'private' stamp: 'tomooda 11/21/2012 13:39'!
addModel: anSIVDMModel 
	^ models add: anSIVDMModel! !

!SIVDMFlatSpecification methodsFor: 'initialize-release' stamp: 'tomooda 11/21/2012 10:57'!
initialize
	super initialize.
	models := WeakRegistry new! !

!SIVDMFlatSpecification methodsFor: 'processing' stamp: 'tomooda 11/21/2012 13:22'!
recompileIfError: errorBlock 
	| vdmj |
	vdmj := SIVDM plain.
	vdmj startupProcess.
	SIFileUtils
		temporaryFile: 'SIVDMFlatSpecification_recompileIfError:-'
		randomSuffixSize: 8
		suffix: '.vdm'
		do: [:name :stream | 
			stream ascii.
			stream lineEndConvention: #lf.
			self printSpecificationOn: stream.
			stream close.
			self send: 'load ' , name.
			self
				expression: self readUpToPrompt
				ifError: [:msg | ^ errorBlock value: msg]].
	vdmj close.
	vdmj := nil.
	models
		do: [:model | model recompile]! !

!SIVDMFlatSpecification methodsFor: 'private' stamp: 'tomooda 11/21/2012 13:39'!
removeModel: anSIVDMModel 
	^ models
		remove: anSIVDMModel
		ifAbsent: []! !

SIVDMDefinition subclass: #SIVDMModule
	instanceVariableNames: 'specification typeExport valueExport functionExport operationExport typeImport valueImport functionImport operationImport'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Specification'!

!SIVDMModule class methodsFor: 'instance creation' stamp: 'tomooda 11/21/2012 14:59'!
in: anSIVDMModuledSpecification 
	^ self new setSpecification: anSIVDMModuledSpecification;
		 yourself! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
functionExport
	^ functionExport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
functionExport: sourceString 
	^ self
		functionExport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:54'!
functionExport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := functionExport.
	functionExport := sourceString.
	self
		recompileIfError: [:msg | 
			functionExport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:46'!
functionImport
	^ functionImport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:48'!
functionImport: sourceString 
	^ self
		functionImport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:54'!
functionImport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := functionImport.
	functionImport := sourceString.
	self
		recompileIfError: [:msg | 
			functionImport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMModule methodsFor: 'initialize-release' stamp: 'tomooda 12/3/2012 10:46'!
initialize
	super initialize.
	typeExport := String new.
	valueExport := String new.
	functionExport := String new.
	operationExport := String new.
	typeImport := String new.
	valueImport := String new.
	functionImport := String new.
	operationImport := String new! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
operationExport
	^ operationExport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
operationExport: sourceString 
	^ self
		operationExport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:54'!
operationExport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := operationExport.
	operationExport := sourceString.
	self
		recompileIfError: [:msg | 
			operationExport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:46'!
operationImport
	^ operationImport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:48'!
operationImport: sourceString 
	^ self
		operationImport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:48'!
operationImport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := operationImport.
	operationImport := sourceString.
	self
		recompileIfError: [:msg | 
			operationImport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMModule methodsFor: 'printing' stamp: 'tomooda 12/3/2012 11:08'!
printSpecificationOn: aStream 
	| imports exports |
	imports := String new writeStream.
	typeImport
		ifNotEmpty: [imports nextPutAll: 'types ';
				 nextPutAll: typeImport;
				 nextPutAll: ';';
				 cr].
	valueImport
		ifNotEmpty: [imports nextPutAll: 'values ';
				 nextPutAll: valueImport;
				 nextPutAll: ';';
				 cr].
	functionImport
		ifNotEmpty: [imports nextPutAll: 'functions ';
				 nextPutAll: functionImport;
				 nextPutAll: ';';
				 cr].
	operationImport
		ifNotEmpty: [imports nextPutAll: 'operations ';
				 nextPutAll: operationImport;
				 nextPutAll: ';';
				 cr].
	imports contents
		ifNotEmpty: [:importSource | 
			aStream nextPutAll: 'imports';
				 cr.
			aStream nextPutAll: importSource;
				 cr].
	exports := String new writeStream.
	typeExport
		ifNotEmpty: [exports nextPutAll: 'types ';
				 nextPutAll: typeExport;
				 nextPutAll: ';';
				 cr].
	valueExport
		ifNotEmpty: [exports nextPutAll: 'values ';
				 nextPutAll: valueExport;
				 nextPutAll: ';';
				 cr].
	functionExport
		ifNotEmpty: [exports nextPutAll: 'functions ';
				 nextPutAll: functionExport;
				 nextPutAll: ';';
				 cr].
	operationExport
		ifNotEmpty: [exports nextPutAll: 'operations ';
				 nextPutAll: operationExport;
				 nextPutAll: ';';
				 cr].
	exports contents
		ifNotEmpty: [:exportSource | 
			aStream nextPutAll: 'exports';
				 cr.
			aStream nextPutAll: exportSource;
				 cr].
	aStream nextPutAll: 'definitions';
		 cr.
	super printSpecificationOn: aStream! !

!SIVDMModule methodsFor: 'processing' stamp: 'tomooda 11/21/2012 13:19'!
recompileIfError: errorBlock 
	^ specification recompileIfError: errorBlock! !

!SIVDMModule methodsFor: 'private' stamp: 'tomooda 11/21/2012 14:58'!
setSpecification: anSIVDMModuledSpecification 
	specification := anSIVDMModuledSpecification! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:57'!
specification
	^ specification! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
typeExport
	^ typeExport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
typeExport: sourceString 
	^ self
		typeExport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
typeExport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := typeExport.
	typeExport := sourceString.
	self
		recompileIfError: [:msg | 
			typeExport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:46'!
typeImport
	^ typeImport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:48'!
typeImport: sourceString 
	^ self
		typeImport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:48'!
typeImport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := typeImport.
	typeImport := sourceString.
	self
		recompileIfError: [:msg | 
			typeImport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
valueExport
	^ valueExport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
valueExport: sourceString 
	^ self
		valueExport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:50'!
valueExport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := valueExport.
	valueExport := sourceString.
	self
		recompileIfError: [:msg | 
			valueExport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:46'!
valueImport
	^ valueImport! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:48'!
valueImport: sourceString 
	^ self
		valueImport: sourceString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIVDMModule methodsFor: 'accessing' stamp: 'tomooda 12/3/2012 10:48'!
valueImport: sourceString ifError: errorBlock 
	| oldSpec |
	oldSpec := valueImport.
	valueImport := sourceString.
	self
		recompileIfError: [:msg | 
			valueImport := oldSpec.
			^ errorBlock value: msg].
	^ sourceString! !

Object subclass: #SIVDMEvaluationResult
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-UI'!

!SIVDMEvaluationResult class methodsFor: 'instance creation' stamp: 'tomooda 12/5/2012 09:08'!
on: aString 
	^ self new result: aString;
		 yourself! !

!SIVDMEvaluationResult methodsFor: 'accessing' stamp: 'tomooda 12/5/2012 09:09'!
printString
	^ result! !

!SIVDMEvaluationResult methodsFor: 'accessing' stamp: 'tomooda 12/5/2012 09:09'!
result
	^ result! !

!SIVDMEvaluationResult methodsFor: 'accessing' stamp: 'tomooda 12/5/2012 09:09'!
result: aString
	result := aString! !

Object subclass: #SIVDMEvaluator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-UI'!

!SIVDMEvaluator methodsFor: 'evaluating' stamp: 'tomooda 12/5/2012 09:09'!
evaluate: textOrStream in: anSIVDMModule to: anSIVDMInspectiveBrowser notifying: aController ifFail: errorBlock logged: aBoolean 
	^ SIVDMEvaluationResult
		on: (anSIVDMInspectiveBrowser
				evaluate: (textOrStream isStream
						ifTrue: [textOrStream contents]
						ifFalse: [textOrStream asString])
				ifError: [:msg | msg])! !

Object subclass: #SIVDMModuledSpecification
	instanceVariableNames: 'modules models'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Specification'!

!SIVDMModuledSpecification class methodsFor: 'examples' stamp: 'tomooda 12/19/2012 10:50'!
examplePedometer
	"SIVDMModuledSpecification examplePedometer"
	| spec inspectiveBrowser |
	spec := SIVDMModuledSpecification new.
	(spec addModuleNamed: 'Counter') stateAt: 'steps' put: 'state steps of
    count : nat
inv
    mk_steps(count) == count >= 0
init
    init_steps == init_steps = mk_steps(0)
end';
		 operationAt: 'getCount' put: 'getCount: () ==> int
getCount() == return count;';
		 operationAt: 'incCount' put: 'incCount: () ==> ()
incCount() == count := count + 1;';
		 operationAt: 'resetCount' put: 'resetCount: () ==> ()
resetCount() == count := 0;';
		 operationExport: 'getCount:()==>int;incCount:()==>();resetCount:()==>()'.
	inspectiveBrowser := SIVDMInspectiveBrowser inspect: (SIVDM specification: spec specificationString).
	inspectiveBrowser specification: spec. 
	ToolBuilder open: inspectiveBrowser label: 'Pedometer'.
	^ spec! !

!SIVDMModuledSpecification methodsFor: 'private' stamp: 'tomooda 11/21/2012 13:40'!
addModel: anSIVDMModel 
	^ models add: anSIVDMModel! !

!SIVDMModuledSpecification methodsFor: 'adding' stamp: 'tomooda 11/21/2012 15:00'!
addModuleNamed: aString 
	^ (modules includesKey: aString) not
		ifTrue: [self
				moduleAt: aString
				put: (SIVDMModule in: self)
				ifError: [:msg | msg]]! !

!SIVDMModuledSpecification methodsFor: 'initialize-release' stamp: 'tomooda 11/21/2012 11:00'!
initialize
	super initialize.
	modules := Dictionary new.
	models := WeakRegistry new! !

!SIVDMModuledSpecification methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:53'!
moduleAt: aString ifAbsent: errorBlock
	^ modules at: aString ifAbsent: errorBlock! !

!SIVDMModuledSpecification methodsFor: 'accessing' stamp: 'tomooda 11/21/2012 14:56'!
moduleAt: aString put: anSIVDMModule ifError: errorBlock 
	| oldModule |
	oldModule := modules
				at: aString
				ifAbsent: [].
	modules at: aString put: anSIVDMModule.
	self
		recompileIfError: [:msg | 
			oldModule
				ifNil: [modules removeKey: aString]
				ifNotNil: [modules at: aString put: oldModule].
			^ errorBlock value: msg].
	^ anSIVDMModule! !

!SIVDMModuledSpecification methodsFor: 'accessing' stamp: 'tomooda 11/24/2012 10:31'!
moduleNames
	^ modules keys asSortedCollection asArray! !

!SIVDMModuledSpecification methodsFor: 'printing' stamp: 'tomooda 12/3/2012 11:10'!
printSpecificationOn: aStream 
	modules
		keysAndValuesDo: [:moduleName :module | 
			aStream nextPutAll: 'module ';
				 nextPutAll: moduleName;
				 cr.
			module printSpecificationOn: aStream.
			aStream cr.
			aStream nextPutAll: 'end ';
				 nextPutAll: moduleName;
				 cr]! !

!SIVDMModuledSpecification methodsFor: 'processing' stamp: 'tomooda 12/3/2012 11:13'!
recompileIfError: errorBlock 
	| vdmj |
	vdmj := SIVDM plain.
	vdmj startupProcess.
	SIFileUtils
		temporaryFile: 'SIVDMModuledSpecification_recompileIfError:-'
		randomSuffixSize: 8
		suffix: '.vdm'
		do: [:name :stream | 
			stream ascii.
			stream lineEndConvention: #lf.
			self printSpecificationOn: stream.
			stream close.
			vdmj send: 'load ' , name.
			vdmj
				expression: vdmj readUpToPrompt
				ifError: [:msg | ^ errorBlock value: msg]].
	vdmj close.
	vdmj := nil.
	models
		do: [:model | model recompile]! !

!SIVDMModuledSpecification methodsFor: 'private' stamp: 'tomooda 11/21/2012 13:40'!
removeModel: anSIVDMModel 
	^ models
		remove: anSIVDMModel
		ifAbsent: []! !

!SIVDMModuledSpecification methodsFor: 'printing' stamp: 'tomooda 12/3/2012 11:11'!
specificationString
	| stream |
	stream := String new writeStream.
	self printSpecificationOn: stream.
	^ stream contents! !

Object subclass: #SIVDMTokenValue
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Process'!

!SIVDMTokenValue class methodsFor: 'instance creation' stamp: 'TO 7/25/2012 10:15'!
with: anObject 
	^ self new setValue: anObject;
		 yourself! !

!SIVDMTokenValue methodsFor: 'comparing' stamp: 'TO 7/25/2012 15:33'!
= anObject
	^ anObject class == self class and: [anObject value = self value]! !

!SIVDMTokenValue methodsFor: 'comparing' stamp: 'TO 7/25/2012 15:33'!
hash
	^ self class hash bitXor: value hash! !

!SIVDMTokenValue methodsFor: 'printing' stamp: 'TO 7/25/2012 10:16'!
printOn: aStream
	aStream nextPutAll: 'mk_token('.
	value printOn: aStream.
	aStream nextPut: $)! !

!SIVDMTokenValue methodsFor: 'private' stamp: 'TO 7/25/2012 10:16'!
setValue: anObject
	value := anObject! !

!SIVDMTokenValue methodsFor: 'printing' stamp: 'TO 7/25/2012 10:17'!
storeOn: aStream 
	aStream nextPutAll: self class name.
	aStream nextPutAll: ' with: ('.
	value storeOn: aStream.
	aStream nextPut: $)! !

!SIVDMTokenValue methodsFor: 'accessing' stamp: 'TO 7/25/2012 10:15'!
value
	^ value! !

SIValueParser subclass: #SIVDMValueParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-VDM-Process'!

!SIVDMValueParser methodsFor: 'defaults' stamp: 'tomooda 11/1/2012 11:05'!
defaultAlgebraicDataClass
	^ SIVDMRecordValue! !

!SIVDMValueParser methodsFor: 'defaults' stamp: 'tomooda 11/1/2012 10:54'!
defaultClassMap
	^ super defaultClassMap
		at: #Map
		put: [:dict | Dictionary withAll: dict];
		
		at: #Quote
		put: [:string | string asSymbol];
		
		at: #Token
		put: [:value | SIVDMTokenValue with: value];
		
		at: #Record
		put: [:constructor :args | self constructAlgebraicData: constructor args: args];
		 yourself! !

!SIVDMValueParser methodsFor: 'defaults' stamp: 'TomohiroOda 7/20/2012 10:24'!
defaultEscapeMap
	| readHex readOctal |
	readHex := [ :stream | 
	| code |
	code := 0.
	[ 
	stream atEnd not
		and: [ 
			| index |
			index := #($0 $1 $2 $3 $4 $5 $6 $7 $8 $9 $a $b $c $d $e $f) indexOf: stream peek asLowercase.
			index > 0
				and: [ 
					code := code * 16 + index - 1.
					stream next notNil ] ] ] whileTrue.
	Character value: code ].
	readOctal := [ :stream | 
	| code |
	code := 0.
	[ 
	stream atEnd not
		and: [ 
			| index |
			index := #($0 $1 $2 $3 $4 $5 $6 $7) indexOf: stream peek.
			index > 0
				and: [ 
					code := code * 8 + index - 1.
					stream next notNil ] ] ] whileTrue.
	Character value: code ].
	^ (Dictionary new)
		at: $a put: (Character value: 7);
		at: $b put: (Character value: 8);
		at: $f put: (Character value: 12);
		at: $n put: Character lf;
		at: $r put: Character cr;
		at: $t put: Character tab;
		at: $v put: (Character value: 11);
		at: $e put: (Character value: 27);
		at: $' put: $';
		at: $" put: $";
		at: $\ put: $\;
		at: $x put: readHex;
		at: $c
			put: [ :stream | 
					| code |
					code := stream next asLowercase.
					(code between: $a and: $z)
						ifTrue: [ Character value: code - $a + 1 ]
						ifFalse: [ self error: 'Invalid control format in a String literal' ] ];
		at: $u put: readHex;
		at: $0 put: readOctal;
		at: $1 put: readOctal;
		at: $2 put: readOctal;
		at: $3 put: readOctal;
		at: $4 put: readOctal;
		at: $5 put: readOctal;
		at: $6 put: readOctal;
		at: $7 put: readOctal;
		yourself! !

!SIVDMValueParser methodsFor: 'testing' stamp: 'tomooda 11/16/2012 14:20'!
isIdentifierChar: aCharacter
	^ (super isIdentifierChar: aCharacter) or: [ aCharacter = $` ]! !

!SIVDMValueParser methodsFor: 'parsing' stamp: 'tomooda 11/13/2012 14:03'!
parseErrorMessage: aStream
	| message |
	message := String new writeStream.
	[ 
	| char |
	aStream atEnd
		ifTrue: [ ^ message contents ].
	char := aStream next.
	char = Character lf
		ifTrue: [ char := Character cr ].
	message nextPut: char.
	(char = Character cr and: [ aStream peek = $> ])
		ifTrue: [ ^ message contents ] ] repeat! !

!SIVDMValueParser methodsFor: 'parsing' stamp: 'tomooda 2/14/2013 11:46'!
parseIdentifierExpression: aReadStream 
	| constructor args |
	constructor := self parseIdentifier: aReadStream.
	constructor = 'true'
		ifTrue: [^ true].
	constructor = 'false'
		ifTrue: [^ false].
	constructor = 'nil'
		ifTrue: [ ^nil ].
	aReadStream peek = $(
		ifFalse: [^ self error: 'Invalid function expression. Missing (.'].
	aReadStream next.
	self skipSeparator: aReadStream.
	args := aReadStream peek = $)
				ifTrue: [Array new]
				ifFalse: [self parseExpressionList: aReadStream separatedBy: ','].
	self skipSeparator: aReadStream.
	aReadStream next = $) ifFalse: [^self error: 'Invalid function expression. Missing ).'].
	constructor = 'mk_token'
		ifTrue: [^ (classMap at: #Token)
				value: args first].
	constructor = 'mk_'
		ifTrue: [^ (classMap at: #Tuple)
				value: args].
	^ (classMap at: #Record)
		value: constructor
		value: args! !

!SIVDMValueParser methodsFor: 'parsing' stamp: 'TO 9/20/2012 11:22'!
parseParen: aReadStream
	| value |
	aReadStream next = $(
		ifFalse: [ ^ self error: 'Missing (' ].
	aReadStream peek = $)
		ifTrue: [ aReadStream next. ^nil].
	value := self parseStream: aReadStream.
	aReadStream next = $)
		ifFalse: [ ^ self error: 'Missing )' ].
	^ value! !

!SIVDMValueParser methodsFor: 'parsing' stamp: 'TO 7/25/2012 10:12'!
parseQuote: aReadStream
	| value |
	aReadStream next = $<
		ifFalse: [ ^ self error: 'Missing <' ].
	value := String new writeStream.
	[aReadStream peek ~= $>] whileTrue: [
		value nextPut: aReadStream next].
	aReadStream next = $>
		ifFalse: [ ^ self error: 'Missing >' ].
	^ value contents asSymbol! !

!SIVDMValueParser methodsFor: 'parsing' stamp: 'TO 7/25/2012 15:25'!
parseSet: aReadStream
	| maps expression char |
	aReadStream next = ${
		ifFalse: [ ^ self error: 'Invalid expression. Missing {' ].
	aReadStream peek = $}
		ifTrue: [ 
			aReadStream next.
			^ (classMap at: #Set) value: Array new ].
	expression := self parseStream: aReadStream.
	char := aReadStream peek.
	char = $}
		ifTrue: [ 
			aReadStream next.
			^ (classMap at: #Set) value: (Array with: expression) ].
	char = $,
		ifTrue: [ 
			aReadStream next.
			self skipSeparator: aReadStream.
			^ (classMap at: #Set) value: ((self parseExpressionList: aReadStream separatedBy: ',') copyWith: expression) ].
	self skipSeparator: aReadStream.
	(self parse: aReadStream accept: '|=>')
		ifFalse: [ ^ self error: 'Invalid map expression. Missing |=> ' ].
	maps := Dictionary new.
	maps at: expression put: (self parseStream: aReadStream).
	char := aReadStream peek.
	char = $}
		ifTrue: [ 
			aReadStream next.
			^ (classMap at: #Map) value: maps ].
	(self parse: aReadStream accept: ',')
		ifFalse: [ ^ self error: 'Invalid map expression. Missing , ' ].
	[ aReadStream atEnd not ]
		whileTrue: [ 
			expression := self parseStream: aReadStream.
			self skipSeparator: aReadStream.
			(self parse: aReadStream accept: '|=>')
				ifFalse: [ ^ self error: 'Invalid map expression. Missing |=> ' ].
			maps at: expression put: (self parseStream: aReadStream).
			char := aReadStream peek.
			char = $}
				ifTrue: [ 
					aReadStream next.
					^ (classMap at: #Map) value: maps ].
			self skipSeparator: aReadStream.
			(self parse: aReadStream accept: ',')
				ifFalse: [ ^ self error: 'Invalid map expression. Missing , ' ] ].
	^ self error: 'Unexpected end of stream. Missing }'! !

!SIVDMValueParser methodsFor: 'parsing' stamp: 'TO 9/20/2012 11:27'!
parseStream: aReadStream ifError: errorBlock
	| char |
	self skipSeparator: aReadStream.
	char := aReadStream peek.
	(char isDigit
			or: [char = $-])
		ifTrue: [^ self parseNumber: aReadStream].
	(self isIdentifierChar: char)
		ifTrue: [^ self parseIdentifierExpression: aReadStream].
	char = $(
		ifTrue: [^ self parseParen: aReadStream].
	char = $[
		ifTrue: [^ self parseList: aReadStream].
	char = ${
		ifTrue: [^ self parseSet: aReadStream].
	char = $<
		ifTrue: [^ self parseQuote: aReadStream].
	char = $"
		ifTrue: [^ self parseString: aReadStream].
	char = $'
		ifTrue: [^ self parseCharacter: aReadStream].
	^ errorBlock value: 'Unknown expression'! !

!SIVDMValueParser methodsFor: 'private' stamp: 'tomooda 11/14/2012 15:18'!
scanTuple: aPositionableStream plainElementsDo: aBlock ifError: errorBlock
	(aPositionableStream next: 4) = 'mk_(' ifFalse: [ ^errorBlock value: 'Not a tuple'].
	[ | start end |
	aPositionableStream skipSeparators.
	start := aPositionableStream position.
	self parseStream: aPositionableStream.
	end := aPositionableStream position.
	aPositionableStream position: start.
	aBlock value: (aPositionableStream next: end - start).
	aPositionableStream next = $,] whileTrue! !
